@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -42,7 +42,7 @@ struct dummy_sched_data
 {
 	int verbose;
 	struct starpu_task_list sched_list;
-     	starpu_pthread_mutex_t policy_mutex;
+	starpu_pthread_mutex_t policy_mutex;
 };
 
 static void dummy_deinit_data(struct starpu_sched_component * component)
@@ -69,7 +69,7 @@ static int dummy_push_task(struct starpu_sched_component *component, struct star
 
 	/* lock all workers when pushing tasks on a list where all
 	   of them would pop for tasks */
-        STARPU_PTHREAD_MUTEX_LOCK(&data->policy_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data->policy_mutex);
 
 	starpu_task_list_push_front(&data->sched_list, task);
 
@@ -198,30 +198,11 @@ static struct starpu_sched_policy dummy_sched_policy =
 	.pop_task = starpu_sched_tree_pop_task,
 	.pre_exec_hook = starpu_sched_component_worker_pre_exec_hook,
 	.post_exec_hook = starpu_sched_component_worker_post_exec_hook,
-	.pop_every_task = NULL,
 	.policy_name = "dummy",
 	.policy_description = "dummy modular scheduling strategy",
 	.worker_type = STARPU_WORKER_LIST,
 };
 
-void dummy_func(void *descr[], void *arg)
-{
-	(void)descr;
-	(void)arg;
-}
-
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.cuda_funcs = {dummy_func},
-        .opencl_funcs = {dummy_func},
-	.model = &starpu_perfmodel_nop,
-	.nbuffers = 0,
-	.name = "dummy",
-};
-
-
 int main(void)
 {
 	int ntasks = NTASKS;
@@ -244,12 +225,14 @@ int main(void)
 	ntasks /= 100;
 #endif
 
+	starpu_codelet_nop.model = &starpu_perfmodel_nop;
+
 	int i;
 	for (i = 0; i < ntasks; i++)
 	{
 		struct starpu_task *task = starpu_task_create();
 
-		task->cl = &dummy_codelet;
+		task->cl = &starpu_codelet_nop;
 		task->cl_arg = NULL;
 
 		ret = starpu_task_submit(task);
