@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2013,2015                           Inria
- * Copyright (C) 2010-2019                                Université de Bordeaux
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -37,14 +35,13 @@
 struct dummy_sched_data
 {
 	struct starpu_task_list sched_list;
-     	starpu_pthread_mutex_t policy_mutex;
+	starpu_pthread_mutex_t policy_mutex;
 };
 
 static void init_dummy_sched(unsigned sched_ctx_id)
 {
 	struct dummy_sched_data *data = (struct dummy_sched_data*)malloc(sizeof(struct dummy_sched_data));
 
-
 	/* Create a linked-list of tasks and a condition variable to protect it */
 	starpu_task_list_init(&data->sched_list);
 
@@ -78,26 +75,23 @@ static int push_task_dummy(struct starpu_task *task)
 
 	/* lock all workers when pushing tasks on a list where all
 	   of them would pop for tasks */
-        STARPU_PTHREAD_MUTEX_LOCK(&data->policy_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data->policy_mutex);
 
 	starpu_task_list_push_front(&data->sched_list, task);
 
 	starpu_push_task_end(task);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data->policy_mutex);
 
-
-        /*if there are no tasks block */
-        /* wake people waiting for a task */
+	/*if there are no tasks block */
+	/* wake people waiting for a task */
 	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
-
-        struct starpu_sched_ctx_iterator it;
-
+	struct starpu_sched_ctx_iterator it;
 	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
-        {
+	{
 		unsigned worker = workers->get_next(workers, &it);
 		starpu_wake_worker_relax_light(worker);
-        }
+	}
 
 	return 0;
 }
@@ -134,24 +128,6 @@ static struct starpu_sched_policy dummy_sched_policy =
 	.worker_type = STARPU_WORKER_LIST,
 };
 
-void dummy_func(void *descr[], void *arg)
-{
-	(void)descr;
-	(void)arg;
-}
-
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.cuda_funcs = {dummy_func},
-        .opencl_funcs = {dummy_func},
-	.model = &starpu_perfmodel_nop,
-	.nbuffers = 0,
-	.name = "dummy",
-};
-
-
 int main(void)
 {
 	int ntasks = NTASKS;
@@ -174,12 +150,14 @@ int main(void)
 	ntasks /= 100;
 #endif
 
+	starpu_codelet_nop.model = &starpu_perfmodel_nop;
+
 	int i;
 	for (i = 0; i < ntasks; i++)
 	{
 		struct starpu_task *task = starpu_task_create();
 
-		task->cl = &dummy_codelet;
+		task->cl = &starpu_codelet_nop;
 		task->cl_arg = NULL;
 
 		ret = starpu_task_submit(task);
