@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -71,7 +71,7 @@ static struct starpu_codelet spmv_cl =
 	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-        .opencl_funcs = {spmv_kernel_opencl},
+	.opencl_funcs = {spmv_kernel_opencl},
 	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 	.nbuffers = 3,
@@ -94,7 +94,7 @@ int main(int argc, char **argv)
 	uint32_t nnz;
 	uint32_t *colind;
 	uint32_t *rowptr;
-	
+
 	/* Input and Output vectors */
 	float *vector_in_ptr;
 	float *vector_out_ptr;
@@ -136,7 +136,7 @@ int main(int argc, char **argv)
 			colind[pos] = row-1;
 			pos++;
 		}
-		
+
 		nzval[pos] = MIDDLE_BAND;
 		colind[pos] = row;
 		pos++;
@@ -152,7 +152,7 @@ int main(int argc, char **argv)
 	STARPU_ASSERT(pos == nnz);
 
 	rowptr[size] = nnz;
-	
+
 	/* initiate the 2 vectors */
 	starpu_malloc((void **)&vector_in_ptr, size*sizeof(float));
 	starpu_malloc((void **)&vector_out_ptr, size*sizeof(float));
@@ -197,11 +197,11 @@ int main(int argc, char **argv)
 	{
 		struct starpu_task *task = starpu_task_create();
 		task->cl = &spmv_cl;
-	
+
 		task->handles[0] = starpu_data_get_sub_data(sparse_matrix, 1, part);
 		task->handles[1] = vector_in;
 		task->handles[2] = starpu_data_get_sub_data(vector_out, 1, part);
-	
+
 		ret = starpu_task_submit(task);
 		if (STARPU_UNLIKELY(ret == -ENODEV))
 		{
@@ -231,7 +231,7 @@ int main(int argc, char **argv)
 	 */
 	for (row = 0; row < STARPU_MIN(size, 16); row++)
 	{
-                FPRINTF(stdout, "%2.2f\t%2.2f\n", vector_in_ptr[row], vector_out_ptr[row]);
+		FPRINTF(stdout, "%2.2f\t%2.2f\n", vector_in_ptr[row], vector_out_ptr[row]);
 	}
 
 	/* Check the result */
@@ -253,12 +253,12 @@ int main(int argc, char **argv)
 		}
 	}
 
-	starpu_free(nzval);
-	starpu_free(colind);
-	starpu_free(rowptr);
-	starpu_free(vector_in_ptr);
-	starpu_free(vector_out_ptr);
-	starpu_free(vector_exp_out_ptr);
+	starpu_free_noflag(nzval, nnz*sizeof(float));
+	starpu_free_noflag(colind, nnz*sizeof(uint32_t));
+	starpu_free_noflag(rowptr, (size+1)*sizeof(uint32_t));
+	starpu_free_noflag(vector_in_ptr, size*sizeof(float));
+	starpu_free_noflag(vector_out_ptr, size*sizeof(float));
+	starpu_free_noflag(vector_exp_out_ptr, size*sizeof(float));
 
 	/*
 	 *	Stop StarPU
