@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,13 +17,12 @@
 #ifndef __STARPU_CUSPARSE_H__
 #define __STARPU_CUSPARSE_H__
 
-#if defined STARPU_USE_CUDA && !defined STARPU_DONT_INCLUDE_CUDA_HEADERS
+#ifdef STARPU_USE_CUDA
 #include <cusparse.h>
 #endif
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -34,21 +33,21 @@ extern "C"
 /**
    Initialize CUSPARSE on every CUDA device
    controlled by StarPU. This call blocks until CUSPARSE has been properly
-   initialized on every device.
+   initialized on every device. See \ref CUDA-specificOptimizations for more details.
 */
 void starpu_cusparse_init(void);
 
 /**
    Synchronously deinitialize the CUSPARSE library on
-   every CUDA device.
+   every CUDA device. See \ref CUDA-specificOptimizations for more details.
 */
 void starpu_cusparse_shutdown(void);
 
-#if defined STARPU_USE_CUDA && !defined STARPU_DONT_INCLUDE_CUDA_HEADERS
+#ifdef STARPU_USE_CUDA
 /**
    Return the CUSPARSE handle to be used to queue CUSPARSE
    kernels. It is properly initialized and configured for multistream by
-   starpu_cusparse_init().
+   starpu_cusparse_init(). See \ref CUDA-specificOptimizations for more details.
 */
 cusparseHandle_t starpu_cusparse_get_local_handle(void);
 #endif
