@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,8 +18,7 @@
 #define __STARPU_SCHED_CTX_HYPERVISOR_H__
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -46,7 +45,7 @@ struct starpu_sched_ctx_performance_counters
 
 	/**
 	   Inform the hypervisor that a task executing a specified
-	   number of instructions has been poped from the worker
+	   number of instructions has been popped from the worker
 	*/
 	void (*notify_poped_task)(unsigned sched_ctx_id, int worker);
 
