@@ -33,8 +33,7 @@ extern "C" {
 
 /**
    @defgroup API_Codelet_And_Tasks Codelet And Tasks
-   @brief This section describes the interface to manipulate codelets
-   and tasks.
+   @brief API to manipulate codelets and tasks.
    @{
 */
 
@@ -48,7 +47,7 @@ extern "C" {
 
 /**
   Convert from enum starpu_worker_archtype to worker type mask for use in "where" fields
-  */
+*/
 #define STARPU_WORKER_TO_MASK(worker_archtype) (1ULL << (worker_archtype + 1))
 
 /**
@@ -121,24 +120,26 @@ extern "C" {
 
 /**
    Value to be set in starpu_codelet::cuda_flags to allow asynchronous
-   CUDA kernel execution.
- */
+   CUDA kernel execution. This requires to use the proper CUDA stream,
+   see \ref CUDA-specificOptimizations
+*/
 #define STARPU_CUDA_ASYNC (1 << 0)
 
 /**
    Value to be set in starpu_codelet::hip_flags to allow asynchronous
-   HIP kernel execution.
- */
+   HIP kernel execution. This requires to use the proper HIP stream
+*/
 #define STARPU_HIP_ASYNC (1 << 0)
 
 /**
    Value to be set in starpu_codelet::opencl_flags to allow
-   asynchronous OpenCL kernel execution.
+   asynchronous OpenCL kernel execution. This requires to use proper queueing,
+   see \ref OpenCL-specificOptimizations
 */
 #define STARPU_OPENCL_ASYNC (1 << 0)
 
 /**
-   To be used when the RAM memory node is specified.
+   To be used as memory node number for the main CPU memory node.
 */
 #define STARPU_MAIN_RAM 0
 
@@ -165,6 +166,9 @@ enum starpu_codelet_type
 			*/
 };
 
+/**
+   todo
+*/
 enum starpu_task_status
 {
 	STARPU_TASK_INIT, /**< The task has just been initialized. */
@@ -208,13 +212,13 @@ typedef void (*starpu_max_fpga_func_t)(void **, void *);
 /**
    @ingroup API_Bubble Hierarchical Dags
    Bubble decision function
- */
+*/
 typedef int (*starpu_bubble_func_t)(struct starpu_task *t, void *arg);
 
 /**
    @ingroup API_Bubble Hierarchical Dags
    Bubble DAG generation function
- */
+*/
 typedef void (*starpu_bubble_gen_dag_func_t)(struct starpu_task *t, void *arg);
 
 /**
@@ -528,11 +532,11 @@ struct starpu_codelet
 
 	/**
 	   Is an array of ::starpu_data_access_mode. It describes the
-	   required access modes to the data neeeded by the codelet
+	   required access modes to the data needed by the codelet
 	   (e.g. ::STARPU_RW). The number of entries in this array
 	   must be specified in the field starpu_codelet::nbuffers,
 	   and should not exceed \ref STARPU_NMAXBUFS. If
-	   unsufficient, this value can be set with the configure
+	   insufficient, this value can be set with the configure
 	   option \ref enable-maxbuffers "--enable-maxbuffers".
 	*/
 	enum starpu_data_access_mode modes[STARPU_NMAXBUFS];
@@ -543,7 +547,7 @@ struct starpu_codelet
 	   (e.g. ::STARPU_RW). The number of entries in this array
 	   must be specified in the field starpu_codelet::nbuffers.
 	   This field should be used for codelets having a number of
-	   datas greater than \ref STARPU_NMAXBUFS (see \ref
+	   data greater than \ref STARPU_NMAXBUFS (see \ref
 	   SettingManyDataHandlesForATask). When defining a codelet,
 	   one should either define this field or the field
 	   starpu_codelet::modes defined above.
@@ -575,7 +579,7 @@ struct starpu_codelet
 	   this specifies the memory nodes where each data should be
 	   sent to for task execution. The number of entries in this
 	   array is starpu_codelet::nbuffers. This field should be
-	   used for codelets having a number of datas greater than
+	   used for codelets having a number of data greater than
 	   \ref STARPU_NMAXBUFS (see \ref
 	   SettingManyDataHandlesForATask). When defining a codelet,
 	   one should either define this field or the field
@@ -725,17 +729,19 @@ struct starpu_task
 	*/
 	int nbuffers;
 
-	/* Keep dyn_handles, dyn_interfaces and dyn_modes before the
-	 * equivalent static arrays, so we can detect dyn_handles
-	 * being NULL while nbuffers being bigger that STARPU_NMAXBUFS
-	 * (otherwise the overflow would put a non-NULL) */
+	/**
+	   Keep dyn_handles, dyn_interfaces and dyn_modes before the
+	   equivalent static arrays, so we can detect dyn_handles
+	   being NULL while nbuffers being bigger that STARPU_NMAXBUFS
+	   (otherwise the overflow would put a non-NULL)
+	*/
 
 	/**
 	   Array of ::starpu_data_handle_t. Specify the handles to the
 	   different pieces of data accessed by the task. The number
 	   of entries in this array must be specified in the field
 	   starpu_codelet::nbuffers. This field should be used for
-	   tasks having a number of datas greater than \ref
+	   tasks having a number of data greater than \ref
 	   STARPU_NMAXBUFS (see \ref SettingManyDataHandlesForATask).
 	   When defining a task, one should either define this field
 	   or the field starpu_task::handles defined below.
@@ -744,6 +750,7 @@ struct starpu_task
 	   when using ::STARPU_DATA_ARRAY and alike.
 	*/
 	starpu_data_handle_t *dyn_handles;
+
 	/**
 	   Array of data pointers to the memory node where execution
 	   will happen, managed by the DSM. Is used when the field
@@ -752,6 +759,7 @@ struct starpu_task
 	   This is filled by StarPU.
 	*/
 	void **dyn_interfaces;
+
 	/**
 	   Used only when starpu_codelet::nbuffers is \ref
 	   STARPU_VARIABLE_NBUFFERS.
@@ -760,7 +768,7 @@ struct starpu_task
 	   (e.g. ::STARPU_RW). The number of entries in this array
 	   must be specified in the field starpu_codelet::nbuffers.
 	   This field should be used for codelets having a number of
-	   datas greater than \ref STARPU_NMAXBUFS (see \ref
+	   data greater than \ref STARPU_NMAXBUFS (see \ref
 	   SettingManyDataHandlesForATask).
 	   When defining a codelet, one should either define this
 	   field or the field starpu_task::modes defined below.
@@ -775,7 +783,7 @@ struct starpu_task
 	   different pieces of data accessed by the task. The number
 	   of entries in this array must be specified in the field
 	   starpu_codelet::nbuffers, and should not exceed
-	   \ref STARPU_NMAXBUFS. If unsufficient, this value can be
+	   \ref STARPU_NMAXBUFS. If insufficient, this value can be
 	   set with the configure option \ref enable-maxbuffers
 	   "--enable-maxbuffers".
 
@@ -796,10 +804,10 @@ struct starpu_task
 	   Used only when starpu_codelet::nbuffers is \ref
 	   STARPU_VARIABLE_NBUFFERS.
 	   Array of ::starpu_data_access_mode which describes the
-	   required access modes to the data neeeded by the codelet
+	   required access modes to the data needed by the codelet
 	   (e.g. ::STARPU_RW). The number of entries in this array
 	   must be specified in the field starpu_task::nbuffers, and
-	   should not exceed \ref STARPU_NMAXBUFS. If unsufficient,
+	   should not exceed \ref STARPU_NMAXBUFS. If insufficient,
 	   this value can be set with the configure option
 	   \ref enable-maxbuffers "--enable-maxbuffers".
 
@@ -865,7 +873,7 @@ struct starpu_task
 	   Optional field. The buffer of starpu_codelet_pack_arg()
 	   and starpu_codelet_unpack_arg() can be allocated with
 	   the starpu_task::cl_ret_size bytes starting at address starpu_task::cl_ret.
-	   starpu_task::cl_ret_size can be used for MPI supoort.
+	   starpu_task::cl_ret_size can be used for MPI support.
 	*/
 	size_t cl_ret_size;
 
@@ -1401,7 +1409,7 @@ struct starpu_task
 	/**
 	   @private
 	   This is private to StarPU, do not modify.
-	 */
+	*/
 	struct starpu_omp_task *omp_task;
 #else
 	void *omp_task;
@@ -1410,25 +1418,25 @@ struct starpu_task
 	/**
 	   When using hierarchical dags, the job identifier of the
 	   bubble task which created the current task
-	 */
+	*/
 	unsigned long bubble_parent;
 
 	/**
 	   When using hierarchical dags, a pointer to the bubble
 	   decision function
-	 */
+	*/
 	starpu_bubble_func_t bubble_func;
 
 	/**
 	   When using hierarchical dags, a pointer to an argument to
 	   be given when calling the bubble decision function
-	 */
+	*/
 	void *bubble_func_arg;
 
 	/**
 	   When using hierarchical dags, a pointer to the bubble
 	   DAG generation function
-	 */
+	*/
 	starpu_bubble_gen_dag_func_t bubble_gen_dag_func;
 
 	/**
@@ -1440,7 +1448,7 @@ struct starpu_task
 	/**
 	   @private
 	   This is private to StarPU, do not modify.
-	  */
+	*/
 	unsigned nb_termination_call_required;
 
 	/**
@@ -1468,13 +1476,13 @@ struct starpu_task
 */
 #define STARPU_TASK_TYPE_DATA_ACQUIRE (1 << 1)
 
+/* Note: remember to update starpu_task_init and starpu_task_ft_create_retry
+ * as well */
 /**
    Value to be used to initialize statically allocated tasks. This is
    equivalent to initializing a structure starpu_task
    with the function starpu_task_init().
 */
-/* Note: remember to update starpu_task_init and starpu_task_ft_create_retry
- * as well */
 #define STARPU_TASK_INITIALIZER                                         \
 	{                                                               \
 		.cl			      = NULL,                   \
@@ -1633,7 +1641,7 @@ struct starpu_task
    will either set the \p i -th element of the field
    starpu_codelet::nodes or the \p i -th element of the field
    starpu_codelet::dyn_nodes (see \ref SettingManyDataHandlesForATask)
- */
+*/
 #define STARPU_CODELET_SET_NODE(codelet, __node, i)       \
 	do {                                              \
 		if ((codelet)->dyn_nodes)                 \
@@ -1685,7 +1693,7 @@ struct starpu_task *starpu_task_create(void) STARPU_ATTRIBUTE_MALLOC;
    with mode \p mode. This allows to synchronize with the task graph, according
    to the sequential consistency, against tasks submitted before or after
    submitting this task. One can then use starpu_task_declare_deps_array() or
-   starpu_task_end_dep_add()/starpu_task_end_dep_release() to add dependencies
+   starpu_task_end_dep_add() / starpu_task_end_dep_release() to add dependencies
    against this task before submitting it.
    See \ref SynchronizationTasks for more details.
  */
@@ -1858,7 +1866,7 @@ void starpu_iteration_push(unsigned long iteration);
 */
 void starpu_iteration_pop(void);
 /**
-	See \ref GraphScheduling for more details.
+   See \ref GraphScheduling for more details.
 */
 void starpu_do_schedule(void);
 
@@ -1913,7 +1921,7 @@ const char *starpu_task_get_name(struct starpu_task *task);
 
 /**
    Allocate a task structure which is the exact duplicate of \p task.
-   See \ref Helpers for more details.
+   See \ref OtherTaskUtility for more details.
 */
 struct starpu_task *starpu_task_dup(struct starpu_task *task);
 
@@ -1935,13 +1943,13 @@ unsigned starpu_task_get_implementation(struct starpu_task *task);
    Create and submit an empty task that unlocks a tag once all its
    dependencies are fulfilled.
    See \ref SynchronizationTasks for more details.
- */
+*/
 void starpu_create_sync_task(starpu_tag_t sync_tag, unsigned ndeps, starpu_tag_t *deps, void (*callback)(void *), void *callback_arg);
 
 /**
    Create and submit an empty task with the given callback.
    See \ref SynchronizationTasks for more details.
- */
+*/
 void starpu_create_callback_task(void (*callback)(void *), void *callback_arg);
 
 /**
@@ -1956,7 +1964,7 @@ void starpu_create_callback_task(void (*callback)(void *), void *callback_arg);
    try-task.
 
    See \ref TaskRetry for more details.
- */
+*/
 void starpu_task_ft_prologue(void *check_ft);
 
 /**
@@ -1967,14 +1975,14 @@ void starpu_task_ft_prologue(void *check_ft);
    try-task as template task.
 
    \p check_ft is similar to the prologue argument of
-   starpu_task_ft_prologue(), and is typicall set to the very function calling
+   starpu_task_ft_prologue(), and is typically set to the very function calling
    starpu_task_ft_create_retry().
 
    The try-task is returned, and can be modified (e.g. to change scheduling
    parameters) before being submitted with starpu_task_submit_nodeps().
 
    See \ref TaskRetry for more details.
- */
+*/
 struct starpu_task *starpu_task_ft_create_retry(const struct starpu_task *meta_task, const struct starpu_task *template_task, void (*check_ft)(void *));
 
 /**
@@ -1985,27 +1993,27 @@ struct starpu_task *starpu_task_ft_create_retry(const struct starpu_task *meta_t
    recorded for performance models.
 
    This can only be called for a task whose data access modes are either
-   STARPU_R and STARPU_W.
- */
+   ::STARPU_R and ::STARPU_W.
+*/
 void starpu_task_ft_failed(struct starpu_task *task);
 
 /**
    Notify that the try-task was successful and thus the meta-task was
    successful.
    See \ref TaskRetry for more details.
- */
+*/
 void starpu_task_ft_success(struct starpu_task *meta_task);
 
 /**
    Set the function to call when the watchdog detects that StarPU has
-   not finished any task for STARPU_WATCHDOG_TIMEOUT seconds.
+   not finished any task for \ref STARPU_WATCHDOG_TIMEOUT seconds.
    See \ref WatchdogSupport for more details.
 */
 void starpu_task_watchdog_set_hook(void (*hook)(void *), void *hook_arg);
 
 /**
- * Return the given status as a string
- */
+   Return the given status as a string
+*/
 char *starpu_task_status_get_as_string(enum starpu_task_status status);
 
 /**
@@ -2039,19 +2047,19 @@ void starpu_set_limit_max_submitted_tasks(int limit_min);
    @return A pointer to an initializes <c>struct starpu_transaction</c>
    or \c NULL if submitting the transaction begin task failed with \c ENODEV.
    See \ref TransactionsCreation for more details.
- */
+*/
 struct starpu_transaction *starpu_transaction_open(int (*do_start_func)(void *buffer, void *arg), void *do_start_arg);
 
 /**
    Function to mark the end of the current transaction epoch and start a new epoch.
    See \ref TransactionsEpochNext for more details.
- */
+*/
 void starpu_transaction_next_epoch(struct starpu_transaction *p_trs, void *do_start_arg);
 
 /**
-   Function to mark the end of the last transaction epoch and free the transation object.
+   Function to mark the end of the last transaction epoch and free the transaction object.
    See \ref TransactionsClosing for more details.
- */
+*/
 void starpu_transaction_close(struct starpu_transaction *p_trs);
 
 /** @} */
