@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -129,7 +129,7 @@ int load_data_wakeup_cond(starpu_data_handle_t handle)
 	return (ld_interface->wakeup_task_threshold > 0) && (ld_interface->nfinished_tasks == ld_interface->wakeup_task_threshold);
 }
 
-static void load_data_register_data_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void load_data_register_data_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	(void) home_node;
 	struct load_data_interface *ld_interface = (struct load_data_interface *) data_interface;
@@ -191,8 +191,7 @@ static int load_data_pack_data(starpu_data_handle_t handle, unsigned node, void
 	*count = load_data_get_size(handle);
 	if (ptr != NULL)
 	{
-		char *data;
-		starpu_malloc_flags((void**) &data, *count, 0);
+		char *data = (void*) starpu_malloc_on_node_flags(node, *count, 0);
 		*ptr = data;
 		memcpy(data, ld_interface, *count);
 	}
@@ -200,7 +199,7 @@ static int load_data_pack_data(starpu_data_handle_t handle, unsigned node, void
 	return 0;
 }
 
-static int load_data_unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int load_data_peek_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	char *data = ptr;
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
@@ -214,6 +213,14 @@ static int load_data_unpack_data(starpu_data_handle_t handle, unsigned node, voi
 	return 0;
 }
 
+static int load_data_unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	load_data_peek_data(handle, node, ptr, count);
+	starpu_free_on_node_flags(node, (uintptr_t) ptr, count, 0);
+
+	return 0;
+}
+
 static int copy_any_to_any(void *src_interface, unsigned src_node,
 			   void *dst_interface, unsigned dst_node,
 			   void *async_data)
@@ -244,6 +251,7 @@ static struct starpu_data_interface_ops interface_load_data_ops =
 	.interface_size = sizeof(struct load_data_interface),
 	.to_pointer = NULL,
 	.pack_data = load_data_pack_data,
+	.peek_data = load_data_peek_data,
 	.unpack_data = load_data_unpack_data,
 	.describe = NULL
 };
@@ -262,11 +270,6 @@ void load_data_data_register(starpu_data_handle_t *handleptr, unsigned home_node
 		.wakeup_ratio = wakeup_ratio
 	};
 
-	if (interface_load_data_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
-	{
-		interface_load_data_ops.interfaceid = starpu_data_interface_get_next_id();
-	}
-
 	starpu_data_register(handleptr, home_node, &load_data, &interface_load_data_ops);
 }
 
