@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +28,9 @@
 #ifdef STARPU_USE_CUDA
 extern void increment_cuda(void *descr[], void *_args);
 #endif
+#ifdef STARPU_USE_HIP
+extern void increment_hip(void *descr[], void *_args);
+#endif
 
 void increment_cpu(void *descr[], void *_args)
 {
@@ -40,6 +43,9 @@ static struct starpu_codelet increment_cl =
 {
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {increment_cuda},
+#endif
+#ifdef STARPU_USE_HIP
+	.hip_funcs = {increment_hip},
 #endif
 	.cpu_funcs = {increment_cpu},
 	.nbuffers = 1,
@@ -63,22 +69,28 @@ int main(int argc, char **argv)
 	int ret, rank, size;
 	int token = 42;
 	starpu_data_handle_t token_handle;
+	int mpi_init;
 
-	ret = starpu_mpi_init_conf(&argc, &argv, 1, MPI_COMM_WORLD, NULL);
+	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED, &mpi_init);
+
+	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, NULL);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
+
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
 
-	if (size < 2 || (starpu_cpu_worker_get_count() + starpu_cuda_worker_get_count() == 0))
+	if (size < 2 || (starpu_cpu_worker_get_count() + starpu_cuda_worker_get_count() == 0) || (starpu_cpu_worker_get_count() + starpu_hip_worker_get_count() == 0))
 	{
 		if (rank == 0)
 		{
 			if (size < 2)
 				FPRINTF(stderr, "We need at least 2 processes.\n");
 			else
-				FPRINTF(stderr, "We need at least 1 CPU or CUDA worker.\n");
+				FPRINTF(stderr, "We need at least 1 CPU or CUDA or HIP worker.\n");
 		}
 		starpu_mpi_shutdown();
+		if (!mpi_init)
+			MPI_Finalize();
 		return rank == 0 ? STARPU_TEST_SKIPPED : 0;
 	}
 
@@ -103,7 +115,8 @@ int main(int argc, char **argv)
 		}
 		else
 		{
-			starpu_mpi_irecv_detached(token_handle, (rank+size-1)%size, tag, MPI_COMM_WORLD, NULL, NULL);
+			ret = starpu_mpi_irecv_detached(token_handle, (rank+size-1)%size, tag, MPI_COMM_WORLD, NULL, NULL);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv_detached");
 		}
 
 		increment_token(token_handle);
@@ -116,14 +129,18 @@ int main(int argc, char **argv)
 		}
 		else
 		{
-			starpu_mpi_isend_detached(token_handle, (rank+1)%size, tag+1, MPI_COMM_WORLD, NULL, NULL);
+			ret = starpu_mpi_isend_detached(token_handle, (rank+1)%size, tag+1, MPI_COMM_WORLD, NULL, NULL);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend_detached");
 		}
 	}
 
 	starpu_task_wait_for_all();
 
 	starpu_data_unregister(token_handle);
+
 	starpu_mpi_shutdown();
+	if (!mpi_init)
+		MPI_Finalize();
 
 #ifndef STARPU_SIMGRID
 	if (rank == last_rank)
