@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Joris Pablo
  * Copyright (C) 2018,2020  Federal University of Rio Grande do Sul (UFRGS)
  *
@@ -247,6 +247,9 @@
 #define _STARPU_FUT_TASK_BUBBLE		0x5189
 #endif
 
+#define	_STARPU_FUT_START_PARALLEL_SYNC	0x518a
+#define	_STARPU_FUT_END_PARALLEL_SYNC	0x518b
+
 /* Predefined FUT key masks */
 #define _STARPU_FUT_KEYMASK_META           FUT_KEYMASK0
 #define _STARPU_FUT_KEYMASK_USER           FUT_KEYMASK1
@@ -841,7 +844,7 @@ do {									\
 				/* Regarding the memory location:
 				 * - if the data interface doesn't provide to_pointer operation, NULL will be returned
 				 *   and the location will be -1, which is fine;
-				 * - we have to check wether the memory is on an actual NUMA node (and not on GPU
+				 * - we have to check whether the memory is on an actual NUMA node (and not on GPU
 				 *   memory, for instance);
 				 * - looking at memory location before executing the task isn't the best choice:
 				 *   the page can be not allocated yet. A solution would be to get the memory
@@ -869,11 +872,17 @@ do {									\
     } \
 } while(0)
 
-#define _STARPU_TRACE_START_EXECUTING()				\
-	FUT_FULL_PROBE1(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_START_EXECUTING, _starpu_gettid());
+#define _STARPU_TRACE_START_EXECUTING(job)				\
+	FUT_FULL_PROBE2(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_START_EXECUTING, _starpu_gettid(), (job)->job_id);
 
-#define _STARPU_TRACE_END_EXECUTING()				\
-	FUT_FULL_PROBE1(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_END_EXECUTING, _starpu_gettid());
+#define _STARPU_TRACE_END_EXECUTING(job)				\
+	FUT_FULL_PROBE2(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_END_EXECUTING, _starpu_gettid(), (job)->job_id);
+
+#define _STARPU_TRACE_START_PARALLEL_SYNC(job)				\
+	FUT_FULL_PROBE2(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_START_PARALLEL_SYNC, _starpu_gettid(), (job)->job_id);
+
+#define _STARPU_TRACE_END_PARALLEL_SYNC(job)				\
+	FUT_FULL_PROBE2(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_END_PARALLEL_SYNC, _starpu_gettid(), (job)->job_id);
 
 #define _STARPU_TRACE_START_CALLBACK(job)	\
 	FUT_FULL_PROBE2(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_START_CALLBACK, job, _starpu_gettid());
@@ -1161,7 +1170,7 @@ do {										\
 #define _STARPU_TRACE_PAPI_TASK_EVENT(event_id, task, value)	\
 	FUT_DO_PROBE3(_STARPU_FUT_PAPI_TASK_EVENT_VALUE, event_id, _starpu_get_job_associated_to_task(task)->job_id, value)
 
-/* We skip these events becasue they are called so often that they cause FxT to
+/* We skip these events because they are called so often that they cause FxT to
  * fail and make the overall trace unreadable anyway. */
 #define _STARPU_TRACE_START_PROGRESS(memnode)		\
 	FUT_FULL_PROBE2(_STARPU_FUT_KEYMASK_WORKER_VERBOSE, _STARPU_FUT_START_PROGRESS_ON_TID, memnode, _starpu_gettid());
@@ -1431,8 +1440,10 @@ do {										\
 #define _STARPU_TRACE_WORKER_INIT_END(workerid)		do {(void)(workerid);} while(0)
 #define _STARPU_TRACE_START_CODELET_BODY(job, nimpl, perf_arch, workerid) 	do {(void)(job); (void)(nimpl); (void)(perf_arch); (void)(workerid);} while(0)
 #define _STARPU_TRACE_END_CODELET_BODY(job, nimpl, perf_arch, workerid)		do {(void)(job); (void)(nimpl); (void)(perf_arch); (void)(workerid);} while(0)
-#define _STARPU_TRACE_START_EXECUTING()		do {} while(0)
-#define _STARPU_TRACE_END_EXECUTING()		do {} while(0)
+#define _STARPU_TRACE_START_EXECUTING(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_END_EXECUTING(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_START_PARALLEL_SYNC(job)	do {(void)(job);} while(0)
+#define _STARPU_TRACE_END_PARALLEL_SYNC(job)	do {(void)(job);} while(0)
 #define _STARPU_TRACE_START_CALLBACK(job)	do {(void)(job);} while(0)
 #define _STARPU_TRACE_END_CALLBACK(job)		do {(void)(job);} while(0)
 #define _STARPU_TRACE_JOB_PUSH(task, prio)	do {(void)(task); (void)(prio);} while(0)
