@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -46,6 +46,9 @@
  *   * Initializes a list (initially empty)
  *   void		FOO_list_init(struct FOO_list*);
  *
+ *   * Initializes a list (initially empty), assuming that the content of FOO_list was already zeroed
+ *   void		FOO_list_init0(struct FOO_list*);
+ *
  *   * Suppresses a liste
  *   void		FOO_list_delete(struct FOO_list*);
  *
@@ -100,7 +103,7 @@
  *   * Return the last element of the list (from the back)
  *   struct FOO*	FOO_list_last(struct FOO_list*);
  *
- *   * Return the value to be tested at the beginning of the list (at the fromt)
+ *   * Return the value to be tested at the beginning of the list (at the front)
  *   struct FOO*	FOO_list_alpha(struct FOO_list*);
  *
  *   * Return the previous element of the list (from the back)
@@ -140,12 +143,12 @@
  *  struct my_struct * e = my_struct_new();
  *  e->a = 0;
  *  e->b = 0;
- *  my_struct_list_push_front(l, e);
+ *  my_struct_list_push_front(&l, e);
  *
  *  - iterating over a list from the front:
  *  struct my_struct * i;
- *  for(i  = my_struct_list_begin(l);
- *      i != my_struct_list_end(l);
+ *  for(i  = my_struct_list_begin(&l);
+ *      i != my_struct_list_end(&l);
  *      i  = my_struct_list_next(i))
  *  {
  *    printf("a=%d; b=%d\n", i->a, i->b);
@@ -153,8 +156,8 @@
  *
  *  - iterating over a list from the back:
  *  struct my_struct * i;
- *  for(i  = my_struct_list_last(l);
- *      i != my_struct_list_alpha(l);
+ *  for(i  = my_struct_list_last(&l);
+ *      i != my_struct_list_alpha(&l);
  *      i  = my_struct_list_prev(i))
  *  {
  *    printf("a=%d; b=%d\n", i->a, i->b);
@@ -220,7 +223,9 @@
   /** @internal */LIST_INLINE struct ENAME *ENAME##_list_back(const struct ENAME##_list *l) \
     { return l->_tail; } \
   /** @internal */LIST_INLINE void ENAME##_list_init(struct ENAME##_list *l) \
-    { l->_head=NULL; l->_tail=l->_head; } \
+    { l->_head=NULL; l->_tail=NULL; } \
+  /** @internal */LIST_INLINE void ENAME##_list_init0(struct ENAME##_list *l STARPU_ATTRIBUTE_UNUSED) \
+    { } \
   /** @internal */LIST_INLINE struct ENAME##_list *ENAME##_list_new(void) \
     { struct ENAME##_list *l; _STARPU_MALLOC(l, sizeof(struct ENAME##_list)); \
       ENAME##_list_init(l); return l; } \
@@ -278,6 +283,62 @@
  *   ENAME_multilist_MEMBER
  * - Then one should call MULTILIST_CREATE_INLINES to create the inlines which
  *   manipulate lists for this MEMBER type.
+ *
+ * *********************************************************
+ * Usage example:
+ *
+ *  - initially you'd have:
+ *    struct my_struct
+ *    {
+ *      int a;
+ *      int b;
+ *    };
+ *
+ * - to make multilists of it, we add MULTILIST_CREATE_TYPE calls before, the
+ *    multilist fields, and MULTILIST_CREATE_INLINES calls after::
+ *
+ *    MULTILIST_CREATE_TYPE(my_struct, foo);
+ *    MULTILIST_CREATE_TYPE(my_struct, bar);
+ *
+ *    struct my_struct
+ *    {
+ *      struct my_struct_multilist_foo foo;
+ *      struct my_struct_multilist_bar bar;
+ *      int a;
+ *      int b;
+ *    };
+ *
+ *    MULTILIST_CREATE_INLINES(struct my_struct, my_struct, foo);
+ *    MULTILIST_CREATE_INLINES(struct my_struct, my_struct, bar);
+ *
+ * - creating a new element and initialize the multilist fields:
+ *
+ *   struct my_struct *e = malloc(sizeof(*e));
+ *   my_struct_multilist_init_foo(e);
+ *   my_struct_multilist_init_bar(e);
+ *   e->a = 0;
+ *   e->b = 0;
+ *
+ * - setting up an empty list:
+ *
+ *   struct my_struct_multilist_foo l;
+ *   my_struct_multilist_head_init_foo(&l);
+ *
+ * - add element 'e' at the front of list 'l':
+ *   my_struct_multilist_push_front_foo(&l, e);
+ *
+ * - TODO implementation: popping from the front:
+ *   struct my_struct *i;
+ *   i = my_struct_multilist_front_foo(&l);
+ *
+ * - iterating over a list from the front:
+ *   struct my_struct *i;
+ *  for(i  = my_struct_multilist_begin_foo(&l);
+ *      i != my_struct_multilist_end_foo(&l);
+ *      i  = my_struct_multilist_next_foo(i))
+ *  {
+ *    printf("a=%d; b=%d\n", i->a, i->b);
+ *  }
  */
 
 /* Create the ENAME_multilist_MEMBER, to be used both as head and as member of main element type */
@@ -363,6 +424,26 @@ LIST_INLINE TYPE *ENAME##_multilist_end_##MEMBER(struct ENAME##_multilist_##MEMB
 LIST_INLINE TYPE *ENAME##_multilist_next_##MEMBER(TYPE *e) { \
 	return ENAME##_of_multilist_##MEMBER(e->MEMBER.next); \
 } \
+/* Return the first element of the list.  */ \
+LIST_INLINE TYPE *ENAME##_multilist_front_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  return ENAME##_of_multilist_##MEMBER(head->next); \
+} \
+/* Return the last element of the list.  */ \
+LIST_INLINE TYPE *ENAME##_multilist_back_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  return ENAME##_of_multilist_##MEMBER(head->prev); \
+} \
+\
+/* Return the first element of the list and erase it. */ \
+LIST_INLINE TYPE *ENAME##_multilist_pop_front_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  TYPE *e = ENAME##_multilist_front_##MEMBER(head); \
+      ENAME##_multilist_erase_##MEMBER(head, e); return e; \
+} \
+/* Return the last element of the list and erase it. */ \
+LIST_INLINE TYPE *ENAME##_multilist_pop_back_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  TYPE *e = ENAME##_multilist_back_##MEMBER(head); \
+      ENAME##_multilist_erase_##MEMBER(head, e); return e; \
+} \
+\
 \
  /* Move a list from its head to another head. Passing newhead == NULL allows to detach the list from any head. */ \
 LIST_INLINE void ENAME##_multilist_move_##MEMBER(struct ENAME##_multilist_##MEMBER *head, struct ENAME##_multilist_##MEMBER *newhead) { \
