@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2011       Télécom-SudParis
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2008-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011	    Télécom-SudParis
+ * Copyright (C) 2013	    Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -43,19 +43,14 @@
 #include <common/utils.h>
 #include <common/list.h>
 
-#ifdef STARPU_USE_CUDA
-#include <cuda.h>
-#endif
+#pragma GCC visibility push(hidden)
 
 struct _starpu_worker;
 
 /** codelet function */
 typedef void (*_starpu_cl_func_t)(void **, void *);
 
-#define _STARPU_CPU_MAY_PERFORM(j)	((j)->task->where & STARPU_CPU)
-#define _STARPU_CUDA_MAY_PERFORM(j)      ((j)->task->where & STARPU_CUDA)
-#define _STARPU_OPENCL_MAY_PERFORM(j)	((j)->task->where & STARPU_OPENCL)
-#define _STARPU_MIC_MAY_PERFORM(j)	((j)->task->where & STARPU_MIC)
+#define _STARPU_MAY_PERFORM(j, arch)	((j)->task->where & STARPU_##arch)
 
 struct _starpu_data_descr
 {
@@ -77,16 +72,16 @@ MULTILIST_CREATE_TYPE(_starpu_job, all_submitted)
 /** A job is the internal representation of a task. */
 struct _starpu_job
 {
-	/** Each job is attributed a unique id. */
+	/** Each job is attributed a unique id. This however only defined when recording traces or using jobid-based task breakpoints */
 	unsigned long job_id;
 
 	/** The task associated to that job */
 	struct starpu_task *task;
 
-        /** A task that this will unlock quickly, e.g. we are the pre_sync part
-         * of a data acquisition, and the caller promised that data release will
+	/** A task that this will unlock quickly, e.g. we are the pre_sync part
+	 * of a data acquisition, and the caller promised that data release will
 	 * happen immediately, so that the post_sync task will be started
-         * immediately after. */
+	 * immediately after. */
 	struct _starpu_job *quick_next;
 
 	/** These synchronization structures are used to wait for the job to be
@@ -191,6 +186,9 @@ struct _starpu_job
 	 * */
 	int task_size;
 
+	/** The worker the task is running on (or -1 when not running yet) */
+	int workerid;
+
 	/** In case we have assigned this job to a combined workerid */
 	int combined_workerid;
 
@@ -211,6 +209,11 @@ struct _starpu_job
 	/** Linked-list of all jobs, for debugging */
 	struct _starpu_job_multilist_all_submitted all_submitted;
 #endif
+
+#ifdef STARPU_BUBBLE
+	int already_turned_into_bubble;
+	unsigned is_bubble:1;
+#endif
 };
 
 #ifdef STARPU_DEBUG
@@ -255,6 +258,8 @@ unsigned _starpu_enforce_deps_starting_from_task(struct _starpu_job *j);
 /** When waking up a continuation, we only enforce new task dependencies */
 unsigned _starpu_reenforce_task_deps_and_schedule(struct _starpu_job *j);
 #endif
+
+unsigned _starpu_take_deps_and_schedule(struct _starpu_job *j);
 void _starpu_enforce_deps_notify_job_ready_soon(struct _starpu_job *j, _starpu_notify_job_start_data *data, int tag);
 
 /** Called at the submission of the job */
@@ -271,10 +276,8 @@ size_t _starpu_job_get_data_size(struct starpu_perfmodel *model, struct starpu_p
 struct starpu_task *_starpu_pop_local_task(struct _starpu_worker *worker);
 
 /** Put a task into the pool of tasks that are explicitly attributed to the
- * specified worker. If "back" is set, the task is put at the back of the list.
- * Considering the tasks are popped from the back, this value should be 0 to
- * enforce a FIFO ordering. */
-int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *task, int prio);
+ * specified worker. */
+int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *task);
 
 #define _STARPU_JOB_GET_ORDERED_BUFFER_INDEX(job, i) ((job->dyn_ordered_buffers) ? job->dyn_ordered_buffers[i].index : job->ordered_buffers[i].index)
 #define _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(job, i) ((job->dyn_ordered_buffers) ? job->dyn_ordered_buffers[i].handle : job->ordered_buffers[i].handle)
@@ -291,4 +294,6 @@ int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *t
 
 #define _STARPU_JOB_GET_DEP_SLOTS(job) (((job)->dyn_dep_slots) ? (job)->dyn_dep_slots : (job)->dep_slots)
 
+#pragma GCC visibility pop
+
 #endif // __JOBS_H__
