@@ -663,7 +663,7 @@ static void measure_bandwidth_between_host_and_dev_on_numa_with_opencl(int dev,
 
 static void measure_bandwidth_between_host_and_dev(int dev, struct dev_timing dev_timing_per_numa[STARPU_NMAXDEVS][STARPU_MAXNUMANODES], enum starpu_node_kind type)
 {
-	/* We measure the bandwith between each GPU and each NUMA node */
+	/* We measure the bandwidth between each GPU and each NUMA node */
 	unsigned numa_id;
 	for (numa_id = 0; numa_id < nnumas; numa_id++)
 	{
@@ -874,7 +874,7 @@ static void benchmark_all_memory_nodes(void)
 	for (i = 0; i < nopencl; i++)
 	{
 		_STARPU_DISP("OpenCL %u...\n", i);
-		/* measure bandwith between Host and Device i */
+		/* measure bandwidth between Host and Device i */
 		measure_bandwidth_between_host_and_dev(i, timing_per_numa[STARPU_OPENCL_RAM], STARPU_OPENCL_RAM);
 	}
 #endif
@@ -2158,7 +2158,7 @@ static void update_bandwidth_through(hwloc_obj_t obj, double bandwidth)
 
 /* find_* functions perform the first step: computing maximum bandwidths */
 
-/* Our trafic had to go through the host, go back from target up to the host,
+/* Our traffic had to go through the host, go back from target up to the host,
  * updating uplink downstream bandwidth along the way */
 static void find_platform_backward_path(hwloc_obj_t obj, double bandwidth)
 {
@@ -2418,7 +2418,7 @@ static void emit_pci_link_down(FILE *f, hwloc_obj_t obj)
 /* Go through PCI hub */
 static void emit_pci_link_through(FILE *f, hwloc_obj_t obj)
 {
-	/* We don't care about trafic going through PCI switches */
+	/* We don't care about traffic going through PCI switches */
 	if (obj->type == HWLOC_OBJ_BRIDGE)
 	{
 		if (!obj->name || !strstr(obj->name, "Switch"))
@@ -2432,7 +2432,7 @@ static void emit_pci_link_through(FILE *f, hwloc_obj_t obj)
 	}
 }
 
-/* Our trafic has to go through the host, go back from target up to the host,
+/* Our traffic has to go through the host, go back from target up to the host,
  * using uplink downstream along the way */
 static void emit_platform_backward_path(FILE *f, hwloc_obj_t obj)
 {
@@ -2864,7 +2864,7 @@ static void write_bus_platform_file_content(int version)
 				}
 
 				/* Otherwise, link to another GPU? */
-				for (k = i+1; k < ncuda; k++)
+				for (k = 0; k < ncuda; k++)
 				{
 					if ((int) pci.domain == props[k].pciDomainID
 					 && (int) pci.bus == props[k].pciBusID
@@ -2880,7 +2880,7 @@ static void write_bus_platform_file_content(int version)
 					continue;
 
 				/* No idea what this is */
-				_STARPU_DISP("Warning: NVLink to unknown PCI card %04x:%02x:%02x: %04x", pci.domain, pci.bus, pci.device, pci.pciDeviceId);
+				_STARPU_DISP("Warning: NVLink to unknown PCI card %04x:%02x:%02x: %04x\n", pci.domain, pci.bus, pci.device, pci.pciDeviceId);
 			}
 		}
 
@@ -3177,11 +3177,14 @@ double starpu_transfer_predict(unsigned src_node, unsigned dst_node, size_t size
 	double bandwidth = bandwidth_matrix[src_node][dst_node];
 	double latency = latency_matrix[src_node][dst_node];
 	struct _starpu_machine_topology *topology = &_starpu_get_machine_config()->topology;
-#if 0
 	int busid = starpu_bus_get_id(src_node, dst_node);
+#if 0
 	int direct = starpu_bus_get_direct(busid);
 #endif
-	float ngpus = topology->ndevices[STARPU_CUDA_WORKER]+topology->ndevices[STARPU_OPENCL_WORKER];
+	float ngpus = starpu_bus_get_ngpus(busid);
+	if (ngpus != 1)
+		ngpus = topology->ndevices[STARPU_CUDA_WORKER]+topology->ndevices[STARPU_OPENCL_WORKER];
+
 #ifdef STARPU_DEVEL
 #warning FIXME: ngpus should not be used e.g. for slow disk transfers...
 #endif
@@ -3217,7 +3220,7 @@ void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double band
 		fprintf(stderr, "Data transfer speed for %s (node %u):\n", name, node);
 	}
 
-	/* save bandwith */
+	/* save bandwidth */
 	for(i = 0; i < STARPU_MAXNODES; ++i)
 	{
 		for(j = 0; j < STARPU_MAXNODES; ++j)
