@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2012-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -180,6 +180,10 @@ msg_as_t _starpu_simgrid_get_as_by_name(const char *name)
 
 int _starpu_simgrid_get_nbhosts(const char *prefix)
 {
+#ifdef HAVE_STARPU_SIMGRID_GET_AS_BY_NAME
+	char new_prefix[32+strlen(prefix)];
+#endif
+
 	int ret;
 #ifdef HAVE_SG_HOST_LIST
 	sg_host_t *hosts_list = NULL;
@@ -191,7 +195,6 @@ int _starpu_simgrid_get_nbhosts(const char *prefix)
 	if (_starpu_simgrid_running_smpi())
 	{
 #ifdef HAVE_STARPU_SIMGRID_GET_AS_BY_NAME
-		char new_prefix[32+strlen(prefix)];
 		char name[32];
 		STARPU_ASSERT(starpu_mpi_world_rank);
 		snprintf(name, sizeof(name), STARPU_MPI_AS_PREFIX"%d", starpu_mpi_world_rank());
@@ -1378,6 +1381,12 @@ void _starpu_simgrid_count_ngpus(void)
 				xbt_dynar_push(route_dynar, &route[i]);
 			free(route);
 #endif
+			if (routesize == 1)
+			{
+				/* Direct link! no need to count anything */
+				starpu_bus_set_ngpus(busid, 1);
+				continue;
+			}
 
 			/* If it goes through "Host", do not care, there is no
 			 * direct transfer support */
@@ -1415,7 +1424,7 @@ void _starpu_simgrid_count_ngpus(void)
 			/* Didn't find it ?! */
 			if (through == -1)
 			{
-				_STARPU_DEBUG("Didn't find through-link for %d->%d\n", src, dst);
+				_STARPU_DISP("Didn't find through-link for %d->%d\n", src, dst);
 				continue;
 			}
 
