@@ -23,6 +23,7 @@
 #include <datawizard/copy_driver.h>
 #include <datawizard/write_back.h>
 #include <datawizard/memory_nodes.h>
+#include <datawizard/sort_data_handles.h>
 #include <core/dependencies/data_concurrency.h>
 #include <core/disk.h>
 #include <profiling/profiling.h>
@@ -77,7 +78,7 @@ int _starpu_select_src_node(starpu_data_handle_t handle, unsigned destination)
 		{
 			if (src_node_mask & (1<<i))
 			{
-				double time = starpu_transfer_predict(i, destination, size);
+				double time;
 				unsigned handling_node;
 
 				/* Avoid indirect transfers */
@@ -85,6 +86,7 @@ int _starpu_select_src_node(starpu_data_handle_t handle, unsigned destination)
 				if (!link_supports_direct_transfers(handle, i, destination, &handling_node))
 					continue;
 
+				time = starpu_transfer_predict(i, destination, size);
 				if (_STARPU_IS_ZERO(time))
 				{
 					/* No estimation, will have to revert to dumb strategy */
@@ -712,7 +714,7 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 		struct _starpu_data_replicate *hop_src_replicate;
 		struct _starpu_data_replicate *hop_dst_replicate;
 
-		/* Only the first request is independant */
+		/* Only the first request is independent */
 		unsigned ndeps = (hop == 0)?0:1;
 
 		hop_src_replicate = &handle->per_node[hop_src_node];
@@ -1162,6 +1164,7 @@ int _starpu_fetch_task_input(struct starpu_task *task, struct _starpu_job *j, in
 	unsigned nacquires;
 
 	unsigned index;
+	int indexdup;
 
 	nacquires = 0;
 	for (index = 0; index < nbuffers; index++)
@@ -1182,11 +1185,18 @@ int _starpu_fetch_task_input(struct starpu_task *task, struct _starpu_job *j, in
 
 		struct _starpu_data_replicate *local_replicate;
 
-		if (index && descrs[index-1].handle == descrs[index].handle)
-			/* We have already took this data, skip it. This
-			 * depends on ordering putting writes before reads, see
-			 * _starpu_compar_handles */
-			continue;
+		for (indexdup = (int) index-1; indexdup >= 0; indexdup--)
+		{
+			starpu_data_handle_t handle_dup = descrs[indexdup].handle;
+			if (handle_dup == handle)
+				/* We have already taken this data, skip it. This
+				 * depends on ordering putting writes before reads, see
+				 * _starpu_compar_handles */
+				goto next;
+			if (!_starpu_handles_same_root(handle_dup, handle))
+				/* We are not checking within the same parent any more, no need to continue checking other handles */
+				break;
+		}
 
 		local_replicate = get_replicate(handle, mode, workerid, node);
 
@@ -1218,6 +1228,8 @@ int _starpu_fetch_task_input(struct starpu_task *task, struct _starpu_job *j, in
 		}
 
 		nacquires++;
+	next:
+		;
 	}
 	_starpu_add_worker_status(worker, STATUS_INDEX_WAITING, NULL);
 	if (async)
@@ -1244,15 +1256,24 @@ enomem:
 
 		struct _starpu_data_replicate *local_replicate;
 
-		if (index2 && descrs[index2-1].handle == descrs[index2].handle)
-			/* We have already released this data, skip it. This
-			 * depends on ordering putting writes before reads, see
-			 * _starpu_compar_handles */
-			continue;
+		for (indexdup = (int) index2+1; indexdup < (int) index; indexdup++)
+		{
+			starpu_data_handle_t handle_dup = descrs[indexdup].handle;
+			if (handle_dup == handle)
+				/* We have already released this data, skip it. This
+				 * depends on ordering putting writes before reads, see
+				 * _starpu_compar_handles */
+				goto next2;
+			if (!_starpu_handles_same_root(handle_dup, handle))
+				/* We are not checking within the same parent any more, no need to continue checking other handles */
+				break;
+		}
 
 		local_replicate = get_replicate(handle, mode, workerid, node);
 
 		_starpu_release_data_on_node(handle, 0, STARPU_NONE, local_replicate);
+	next2:
+		;
 	}
 
 	return -1;
@@ -1293,7 +1314,7 @@ void _starpu_fetch_task_input_tail(struct starpu_task *task, struct _starpu_job
 				!(mode & (STARPU_SCRATCH|STARPU_REDUX)) &&
 				(mode & STARPU_R))
 			{
-				/* Allocations or transfer prefetchs should have been done by now and marked
+				/* Allocations or transfer prefetches should have been done by now and marked
 				 * this mc as needed for us.
 				 * Now that we added a reference for the task, we can relieve that.  */
 				/* Note: the replicate might have been evicted in between, thus not 100% sure
@@ -1352,6 +1373,7 @@ void __starpu_push_task_output(struct _starpu_job *j)
 	int workerid = starpu_worker_get_id();
 
 	unsigned index;
+	int indexdup;
 	for (index = 0; index < nbuffers; index++)
 	{
 		starpu_data_handle_t handle = descrs[index].handle;
@@ -1360,11 +1382,19 @@ void __starpu_push_task_output(struct _starpu_job *j)
 
 		struct _starpu_data_replicate *local_replicate = NULL;
 
-		if (index && descrs[index-1].handle == descrs[index].handle)
-			/* We have already released this data, skip it. This
-			 * depends on ordering putting writes before reads, see
-			 * _starpu_compar_handles */
-			continue;
+		for (indexdup = (int) index-1; indexdup >= 0; indexdup--)
+		{
+			starpu_data_handle_t handle_dup = descrs[indexdup].handle;
+			if (handle_dup == handle)
+				/* We have already released this data, skip it. This
+				 * depends on ordering putting writes before reads, see
+				 * _starpu_compar_handles */
+				goto next;
+
+			if (!_starpu_handles_same_root(handle_dup, handle))
+				/* We are not checking within the same parent any more, no need to continue checking other handles */
+				break;
+		}
 
 		if (node != -1)
 			local_replicate = get_replicate(handle, mode, workerid, node);
@@ -1385,6 +1415,8 @@ void __starpu_push_task_output(struct _starpu_job *j)
 			_starpu_spin_unlock(&handle->header_lock);
 			_starpu_release_data_on_node(handle, 0, STARPU_NONE, local_replicate);
 		}
+	next:
+		;
 	}
 
 	if (profiling && task->profiling_info)
@@ -1494,7 +1526,7 @@ static void _starpu_fetch_nowhere_task_input_cb(void *arg)
 }
 
 /* NB : this value can only be an indication of the status of a data
-	at some point, but there is no strong garantee ! */
+	at some point, but there is no strong guarantee ! */
 unsigned starpu_data_is_on_node(starpu_data_handle_t handle, unsigned node)
 {
 	unsigned ret = 0;
