@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2021       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,14 +25,13 @@
 #include <drivers/mpi/driver_mpi_sink.h>
 #include <drivers/mpi/driver_mpi_source.h>
 #include <drivers/mpi/driver_mpi_common.h>
-#include <drivers/mic/driver_mic_source.h>
+#include <drivers/tcpip/driver_tcpip_sink.h>
+#include <drivers/tcpip/driver_tcpip_source.h>
+#include <drivers/tcpip/driver_tcpip_common.h>
 #include <common/fxt.h>
 #include <datawizard/copy_driver.h>
 #include <datawizard/memalloc.h>
-#include <starpu_opencl.h>
-#include <starpu_cuda.h>
 #include <profiling/profiling.h>
-#include <core/disk.h>
 
 #ifdef STARPU_SIMGRID
 #include <core/simgrid.h>
@@ -140,6 +140,35 @@ void starpu_wake_all_blocked_workers(void)
 static unsigned long communication_cnt = 0;
 #endif
 
+int _starpu_copy_interface_any_to_any(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+{
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
+	enum starpu_node_kind dst_kind = starpu_node_get_kind(dst_node);
+
+	int ret = 0;
+	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
+
+	if (!req || starpu_asynchronous_copy_disabled() ||
+		    starpu_asynchronous_copy_disabled_for(src_kind) ||
+		    starpu_asynchronous_copy_disabled_for(dst_kind) ||
+		    !copy_methods->any_to_any)
+	{
+		/* this is not associated to a request so it's synchronous */
+		STARPU_ASSERT_MSG(copy_methods->any_to_any, "Interface <%s> does not define copy_methods->any_to_any", handle->ops->name);
+		copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, NULL);
+	}
+	else
+	{
+		if (dst_kind == STARPU_CPU_RAM)
+			req->async_channel.node_ops = starpu_memory_driver_info[src_kind].ops;
+		else
+			req->async_channel.node_ops = starpu_memory_driver_info[dst_kind].ops;
+		STARPU_ASSERT(copy_methods->any_to_any);
+		ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, &req->async_channel);
+	}
+	return ret;
+}
+
 static int copy_data_1_to_1_generic(starpu_data_handle_t handle,
 				    struct _starpu_data_replicate *src_replicate,
 				    struct _starpu_data_replicate *dst_replicate,
@@ -160,33 +189,20 @@ static int copy_data_1_to_1_generic(starpu_data_handle_t handle,
 
 	return _starpu_simgrid_transfer(handle->ops->get_size(handle), src_node, dst_node, req);
 #else /* !SIMGRID */
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
 	enum starpu_node_kind dst_kind = starpu_node_get_kind(dst_node);
 	void *src_interface = src_replicate->data_interface;
 	void *dst_interface = dst_replicate->data_interface;
 
-#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
-	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
-	if ((src_kind == STARPU_CUDA_RAM) || (dst_kind == STARPU_CUDA_RAM))
+	const struct _starpu_node_ops *src_node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *dst_node_ops = _starpu_memory_node_get_node_ops(dst_node);
+	if (src_node_ops && src_node_ops->copy_interface_to[dst_kind])
 	{
-		unsigned devid;
-		if ((src_kind == STARPU_CUDA_RAM) && (dst_kind == STARPU_CUDA_RAM))
-		{
-			/* GPU-GPU transfer, issue it from the destination */
-			devid = starpu_memory_node_get_devid(dst_node);
-		}
-		else
-		{
-			unsigned node = (dst_kind == STARPU_CUDA_RAM)?dst_node:src_node;
-			devid = starpu_memory_node_get_devid(node);
-		}
-		starpu_cuda_set_device(devid);
+		return src_node_ops->copy_interface_to[dst_kind](handle, src_interface, src_node, dst_interface, dst_node, req);
 	}
-#endif
-
-	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(src_node);
-	if (node_ops && node_ops->copy_interface_to[dst_kind])
+	else if (dst_node_ops && dst_node_ops->copy_interface_from[src_kind])
 	{
-		return node_ops->copy_interface_to[dst_kind](handle, src_interface, src_node, dst_interface, dst_node, req);
+		return dst_node_ops->copy_interface_from[src_kind](handle, src_interface, src_node, dst_interface, dst_node, req);
 	}
 	else
 	{
@@ -195,51 +211,150 @@ static int copy_data_1_to_1_generic(starpu_data_handle_t handle,
 #endif /* !SIMGRID */
 }
 
+static int update_map_generic(starpu_data_handle_t handle,
+				    struct _starpu_data_replicate *src_replicate,
+				    struct _starpu_data_replicate *dst_replicate,
+				    struct _starpu_data_request *req STARPU_ATTRIBUTE_UNUSED)
+{
+	int src_node = src_replicate->memory_node;
+	int dst_node = dst_replicate->memory_node;
+
+	STARPU_ASSERT(src_replicate->refcnt);
+	STARPU_ASSERT(dst_replicate->refcnt);
+
+	STARPU_ASSERT((src_replicate->mapped == dst_node && dst_replicate->allocated)
+			||(src_replicate->allocated && dst_replicate->mapped == src_node));
+
+	void *src_interface = src_replicate->data_interface;
+	void *dst_interface = dst_replicate->data_interface;
+
+	handle->ops->update_map(src_interface, src_node, dst_interface, dst_node);
+
+	return 0;
+}
+
 int STARPU_ATTRIBUTE_WARN_UNUSED_RESULT _starpu_driver_copy_data_1_to_1(starpu_data_handle_t handle,
 									struct _starpu_data_replicate *src_replicate,
 									struct _starpu_data_replicate *dst_replicate,
 									unsigned donotread,
 									struct _starpu_data_request *req,
-									unsigned may_alloc,
-									enum _starpu_is_prefetch prefetch STARPU_ATTRIBUTE_UNUSED)
+									enum _starpu_may_alloc may_alloc,
+									enum starpu_is_prefetch prefetch STARPU_ATTRIBUTE_UNUSED)
 {
 	if (!donotread)
 	{
-		STARPU_ASSERT(src_replicate->allocated);
+		STARPU_ASSERT(src_replicate->allocated || src_replicate->mapped != STARPU_UNMAPPED);
 		STARPU_ASSERT(src_replicate->refcnt);
 	}
 
 	unsigned src_node = src_replicate->memory_node;
 	unsigned dst_node = dst_replicate->memory_node;
 
-	/* first make sure the destination has an allocated buffer */
-	if (!dst_replicate->allocated)
+	if (!dst_replicate->allocated && dst_replicate->mapped == STARPU_UNMAPPED && dst_node != src_node
+			&& handle->ops->map_data
+			&& (_starpu_memory_node_get_mapped(dst_replicate->memory_node) /* || handle wants it */))
 	{
-		if (!may_alloc || _starpu_is_reclaiming(dst_node))
+		/* Memory node which can just map the main memory, try to map.  */
+		if (!handle->ops->map_data(
+				src_replicate->data_interface, src_replicate->memory_node,
+				dst_replicate->data_interface, dst_replicate->memory_node))
+		{
+			dst_replicate->mapped = src_node;
+
+			if (_starpu_node_needs_map_update(dst_node))
+			{
+				/* Driver porters: adding your driver here is
+				   optional, it is only needed when implementing
+				   support for memory mapping */
+				switch (starpu_node_get_kind(dst_node))
+				{
+					case STARPU_OPENCL_RAM:
+					/* OpenCL mappings write access defaults to the device */
+						dst_replicate->map_write = 1;
+						break;
+					case STARPU_CUDA_RAM:
+						dst_replicate->map_write = 0;
+						break;
+					case STARPU_CPU_RAM:
+					default:
+						/* Should not happen */
+						STARPU_ABORT();
+						break;
+				}
+			}
+		}
+	}
+
+	/* first make sure the destination has an allocated buffer */
+	if (!dst_replicate->allocated && dst_replicate->mapped == STARPU_UNMAPPED)
+	{
+		if (may_alloc==_STARPU_DATAWIZARD_DO_NOT_ALLOC || _starpu_is_reclaiming(dst_node))
 			/* We're not supposed to allocate there at the moment */
 			return -ENOMEM;
 
-		int ret_alloc = _starpu_allocate_memory_on_node(handle, dst_replicate, req ? req->prefetch : STARPU_FETCH);
+		int ret_alloc = _starpu_allocate_memory_on_node(handle, dst_replicate, prefetch, may_alloc==_STARPU_DATAWIZARD_ONLY_FAST_ALLOC);
 		if (ret_alloc)
 			return -ENOMEM;
 	}
 
-	STARPU_ASSERT(dst_replicate->allocated);
+	STARPU_ASSERT(dst_replicate->allocated || dst_replicate->mapped != STARPU_UNMAPPED);
 	STARPU_ASSERT(dst_replicate->refcnt);
 
+	/* In the case of a mapped data, we are here requested either
+	 * - because the destination will write to it, and thus needs write
+	 *   access.
+	 * - because the source was modified, and the destination needs to get
+	 *   updated.
+	 * All in all, any data change will actually trigger both.
+	 */
+	if (!donotread && dst_replicate->mapped != STARPU_UNMAPPED)
+	{
+		STARPU_ASSERT(src_replicate->memory_node == dst_replicate->mapped);
+		if (_starpu_node_needs_map_update(dst_node))
+		{
+			/* We need to flush from RAM to the device */
+			if (!dst_replicate->map_write)
+			{
+				update_map_generic(handle, src_replicate, dst_replicate, req);
+				dst_replicate->map_write = 1;
+			}
+		}
+
+		dst_replicate->initialized = 1;
+	}
+
+	else if (!donotread && src_replicate->mapped != STARPU_UNMAPPED)
+	{
+		STARPU_ASSERT(dst_replicate->memory_node == src_replicate->mapped);
+		if (_starpu_node_needs_map_update(src_node))
+		{
+			/* We need to flush from the device to the RAM */
+			if (src_replicate->map_write)
+			{
+				update_map_generic(handle, src_replicate, dst_replicate, req);
+				src_replicate->map_write = 0;
+			}
+		}
+
+		dst_replicate->initialized = 1;
+	}
+
 	/* if there is no need to actually read the data,
 	 * we do not perform any transfer */
-	if (!donotread)
+	else if (!donotread)
 	{
 		unsigned long STARPU_ATTRIBUTE_UNUSED com_id = 0;
 		size_t size = _starpu_data_get_size(handle);
 		_starpu_bus_update_profiling_info((int)src_node, (int)dst_node, size);
 
 #ifdef STARPU_USE_FXT
-		com_id = STARPU_ATOMIC_ADDL(&communication_cnt, 1);
+		if (fut_active)
+		{
+			com_id = STARPU_ATOMIC_ADDL(&communication_cnt, 1);
 
-		if (req)
-			req->com_id = com_id;
+			if (req)
+				req->com_id = com_id;
+		}
 #endif
 
 		dst_replicate->initialized = 1;
@@ -298,12 +413,21 @@ void starpu_interface_end_driver_copy_async(unsigned src_node, unsigned dst_node
 int starpu_interface_copy(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, void *async_data)
 {
 	struct _starpu_async_channel *async_channel = async_data;
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
 	enum starpu_node_kind dst_kind = starpu_node_get_kind(dst_node);
-	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *src_node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *dst_node_ops = _starpu_memory_node_get_node_ops(dst_node);
 
-	if (node_ops && node_ops->copy_data_to[dst_kind])
+	if (src_node_ops && src_node_ops->copy_data_to[dst_kind])
 	{
-		return node_ops->copy_data_to[dst_kind](src, src_offset, src_node,
+		return src_node_ops->copy_data_to[dst_kind](src, src_offset, src_node,
+							     dst, dst_offset, dst_node,
+							     size,
+							     async_channel);
+	}
+	else if (dst_node_ops && dst_node_ops->copy_data_from[src_kind])
+	{
+		return dst_node_ops->copy_data_from[src_kind](src, src_offset, src_node,
 							     dst, dst_offset, dst_node,
 							     size,
 							     async_channel);
@@ -324,8 +448,10 @@ int starpu_interface_copy2d(uintptr_t src, size_t src_offset, unsigned src_node,
 	int ret = 0;
 	unsigned i;
 	struct _starpu_async_channel *async_channel = async_data;
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
 	enum starpu_node_kind dst_kind = starpu_node_get_kind(dst_node);
-	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *src_node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *dst_node_ops = _starpu_memory_node_get_node_ops(dst_node);
 
 	STARPU_ASSERT_MSG(ld_src >= blocksize, "block size %lu is bigger than ld %lu in source", (unsigned long) blocksize, (unsigned long) ld_src);
 	STARPU_ASSERT_MSG(ld_dst >= blocksize, "block size %lu is bigger than ld %lu in destination", (unsigned long) blocksize, (unsigned long) ld_dst);
@@ -336,9 +462,17 @@ int starpu_interface_copy2d(uintptr_t src, size_t src_offset, unsigned src_node,
 					     dst, dst_offset, dst_node,
 					     blocksize * numblocks, async_data);
 
-	if (node_ops && node_ops->copy2d_data_to[dst_kind])
+	if (src_node_ops && src_node_ops->copy2d_data_to[dst_kind])
 		/* Hardware-optimized non-contiguous case */
-		return node_ops->copy2d_data_to[dst_kind](src, src_offset, src_node,
+		return src_node_ops->copy2d_data_to[dst_kind](src, src_offset, src_node,
+							     dst, dst_offset, dst_node,
+							     blocksize,
+							     numblocks, ld_src, ld_dst,
+							     async_channel);
+
+	if (dst_node_ops && dst_node_ops->copy2d_data_from[src_kind])
+		/* Hardware-optimized non-contiguous case */
+		return dst_node_ops->copy2d_data_from[src_kind](src, src_offset, src_node,
 							     dst, dst_offset, dst_node,
 							     blocksize,
 							     numblocks, ld_src, ld_dst,
@@ -365,14 +499,16 @@ int starpu_interface_copy3d(uintptr_t src, size_t src_offset, unsigned src_node,
 	int ret = 0;
 	unsigned i;
 	struct _starpu_async_channel *async_channel = async_data;
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
 	enum starpu_node_kind dst_kind = starpu_node_get_kind(dst_node);
-	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *src_node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *dst_node_ops = _starpu_memory_node_get_node_ops(dst_node);
 
 	STARPU_ASSERT_MSG(ld1_src >= blocksize, "block size %lu is bigger than ld %lu in source", (unsigned long) blocksize, (unsigned long) ld1_src);
 	STARPU_ASSERT_MSG(ld1_dst >= blocksize, "block size %lu is bigger than ld %lu in destination", (unsigned long) blocksize, (unsigned long) ld1_dst);
 
-	STARPU_ASSERT_MSG(ld2_src >= numblocks_1 * ld1_src, "block group size %lu is bigger than group ld %lu in source", (unsigned long) numblocks_1 * ld1_src, (unsigned long) ld2_src);
-	STARPU_ASSERT_MSG(ld2_dst >= numblocks_1 * ld1_dst, "block group size %lu is bigger than group ld %lu in destination", (unsigned long) numblocks_1 * ld1_dst, (unsigned long) ld2_dst);
+	STARPU_ASSERT_MSG(ld2_src >= numblocks_1 * ld1_src, "block group size %lu is bigger than group ld %lu in source", (unsigned long) (numblocks_1 * ld1_src), (unsigned long) ld2_src);
+	STARPU_ASSERT_MSG(ld2_dst >= numblocks_1 * ld1_dst, "block group size %lu is bigger than group ld %lu in destination", (unsigned long) (numblocks_1 * ld1_dst), (unsigned long) ld2_dst);
 
 	if (ld2_src == blocksize * numblocks_1 &&
 	    ld2_dst == blocksize * numblocks_1)
@@ -382,9 +518,18 @@ int starpu_interface_copy3d(uintptr_t src, size_t src_offset, unsigned src_node,
 					     blocksize * numblocks_1 * numblocks_2,
 					     async_data);
 
-	if (node_ops && node_ops->copy3d_data_to[dst_kind])
+	if (src_node_ops && src_node_ops->copy3d_data_to[dst_kind])
 		/* Hardware-optimized non-contiguous case */
-		return node_ops->copy3d_data_to[dst_kind](src, src_offset, src_node,
+		return src_node_ops->copy3d_data_to[dst_kind](src, src_offset, src_node,
+							     dst, dst_offset, dst_node,
+							     blocksize,
+							     numblocks_1, ld1_src, ld1_dst,
+							     numblocks_2, ld2_src, ld2_dst,
+							     async_channel);
+
+	if (dst_node_ops && dst_node_ops->copy3d_data_from[src_kind])
+		/* Hardware-optimized non-contiguous case */
+		return dst_node_ops->copy3d_data_from[src_kind](src, src_offset, src_node,
 							     dst, dst_offset, dst_node,
 							     blocksize,
 							     numblocks_1, ld1_src, ld1_dst,
@@ -418,11 +563,11 @@ int starpu_interface_copy4d(uintptr_t src, size_t src_offset, unsigned src_node,
 	STARPU_ASSERT_MSG(ld1_src >= blocksize, "block size %lu is bigger than ld %lu in source", (unsigned long) blocksize, (unsigned long) ld1_src);
 	STARPU_ASSERT_MSG(ld1_dst >= blocksize, "block size %lu is bigger than ld %lu in destination", (unsigned long) blocksize, (unsigned long) ld1_dst);
 
-	STARPU_ASSERT_MSG(ld2_src >= numblocks_1 * ld1_src, "block group size %lu is bigger than group ld %lu in source", (unsigned long) numblocks_1 * ld1_src, (unsigned long) ld2_src);
-	STARPU_ASSERT_MSG(ld2_dst >= numblocks_1 * ld1_dst, "block group size %lu is bigger than group ld %lu in destination", (unsigned long) numblocks_1 * ld1_dst, (unsigned long) ld2_dst);
+	STARPU_ASSERT_MSG(ld2_src >= numblocks_1 * ld1_src, "block group size %lu is bigger than group ld %lu in source", (unsigned long) (numblocks_1 * ld1_src), (unsigned long) ld2_src);
+	STARPU_ASSERT_MSG(ld2_dst >= numblocks_1 * ld1_dst, "block group size %lu is bigger than group ld %lu in destination", (unsigned long) (numblocks_1 * ld1_dst), (unsigned long) ld2_dst);
 
-	STARPU_ASSERT_MSG(ld3_src >= numblocks_2 * ld2_src, "block group group size %lu is bigger than group group ld %lu in source", (unsigned long) numblocks_2 * ld2_src, (unsigned long) ld3_src);
-	STARPU_ASSERT_MSG(ld3_dst >= numblocks_2 * ld2_dst, "block group group size %lu is bigger than group group ld %lu in destination", (unsigned long) numblocks_2 * ld2_dst, (unsigned long) ld3_dst);
+	STARPU_ASSERT_MSG(ld3_src >= numblocks_2 * ld2_src, "block group group size %lu is bigger than group group ld %lu in source", (unsigned long) (numblocks_2 * ld2_src), (unsigned long) ld3_src);
+	STARPU_ASSERT_MSG(ld3_dst >= numblocks_2 * ld2_dst, "block group group size %lu is bigger than group group ld %lu in destination", (unsigned long) (numblocks_2 * ld2_dst), (unsigned long) ld3_dst);
 
 	if (ld3_src == blocksize * numblocks_1 * numblocks_2 &&
 	    ld3_dst == blocksize * numblocks_1 * numblocks_2)
@@ -448,12 +593,159 @@ int starpu_interface_copy4d(uintptr_t src, size_t src_offset, unsigned src_node,
 	return ret;
 }
 
+uintptr_t starpu_interface_map(uintptr_t src, size_t src_offset, unsigned src_node, unsigned dst_node, size_t size, int *ret)
+{
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
+	const struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
+
+	if (node_ops && node_ops->map[src_kind])
+	{
+		return node_ops->map[src_kind](src, src_offset, src_node, dst_node, size, ret);
+	}
+	else
+	{
+		*ret = -EIO;
+		return 0;
+	}
+}
+
+int starpu_interface_unmap(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, unsigned dst_node, size_t size)
+{
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
+	const struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
+
+	if (node_ops && node_ops->unmap[src_kind])
+	{
+		return node_ops->unmap[src_kind](src, src_offset, src_node, dst, dst_node, size);
+	}
+	else
+	{
+		STARPU_ABORT_MSG("No unmap function defined from node %s to node %s\n", _starpu_node_get_prefix(starpu_node_get_kind(src_node)), _starpu_node_get_prefix(starpu_node_get_kind(dst_node)));
+		return -1;
+	}
+}
+
+int starpu_interface_update_map(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size)
+{
+	enum starpu_node_kind src_kind = starpu_node_get_kind(src_node);
+	enum starpu_node_kind dst_kind = starpu_node_get_kind(dst_node);
+	const struct _starpu_node_ops *src_node_ops = _starpu_memory_node_get_node_ops(src_node);
+	const struct _starpu_node_ops *dst_node_ops = _starpu_memory_node_get_node_ops(dst_node);
+
+	if (src_node_ops && src_node_ops->update_map[dst_kind])
+	{
+		return src_node_ops->update_map[dst_kind](src, src_offset, src_node, dst, dst_offset, dst_node, size);
+	}
+	else if (dst_node_ops && dst_node_ops->update_map[src_kind])
+	{
+		return dst_node_ops->update_map[src_kind](src, src_offset, src_node, dst, dst_offset, dst_node, size);
+	}
+	else
+	{
+		STARPU_ABORT_MSG("No unmap function defined from node %s to node %s\n", _starpu_node_get_prefix(starpu_node_get_kind(src_node)), _starpu_node_get_prefix(starpu_node_get_kind(dst_node)));
+		return -1;
+	}
+}
+
+static size_t _get_size(uint32_t* nn, size_t ndim)
+{
+	size_t size = 1;
+	unsigned i;
+	for (i=0; i<ndim; i++)
+		size *= nn[i];
+
+	return size;
+}
+
+int starpu_interface_copynd(uintptr_t src, size_t src_offset, unsigned src_node,
+			    uintptr_t dst, size_t dst_offset, unsigned dst_node,
+			    size_t elemsize, size_t ndim,
+			    uint32_t* nn, uint32_t* ldn_src, uint32_t* ldn_dst,
+			    void *async_data)
+{
+	int ret = 0;
+	unsigned i;
+
+	if (ndim > 0)
+	{
+		for (i = 0; i < ndim-1; i++)
+		{
+			STARPU_ASSERT_MSG(ldn_src[i+1] >= nn[i] * ldn_src[i], "block size %lu is bigger than ld %lu in source", (unsigned long) nn[i] * ldn_src[i], (unsigned long) ldn_src[i+1]);
+			STARPU_ASSERT_MSG(ldn_dst[i+1] >= nn[i] * ldn_dst[i], "block size %lu is bigger than ld %lu in destination", (unsigned long) nn[i] * ldn_dst[i], (unsigned long) ldn_dst[i+1]);
+		}
+
+		if (ldn_src[ndim-1] == _get_size(nn, ndim-1) &&
+		    ldn_dst[ndim-1] == _get_size(nn, ndim-1))
+			/* Optimize contiguous case */
+			return starpu_interface_copy(src, src_offset, src_node,
+						     dst, dst_offset, dst_node,
+						     _get_size(nn, ndim) * elemsize,
+						     async_data);
+	}
+
+	if(ndim > 4)
+	{
+		for (i = 0; i < nn[ndim-1]; i++)
+		{
+			if (starpu_interface_copynd(src, src_offset + i*ldn_src[ndim-1]*elemsize, src_node,
+						    dst, dst_offset + i*ldn_dst[ndim-1]*elemsize, dst_node,
+						    elemsize, ndim-1,
+						    nn, ldn_src, ldn_dst,
+						    async_data))
+				ret = -EAGAIN;
+		}
+	}
+	else if(ndim == 4)
+	{
+		return starpu_interface_copy4d(src, src_offset, src_node,
+				    dst, dst_offset, dst_node,
+				    nn[0] * elemsize,
+				    nn[1], ldn_src[1] * elemsize, ldn_dst[1] * elemsize,
+				    nn[2], ldn_src[2] * elemsize, ldn_dst[2] * elemsize,
+				    nn[3], ldn_src[3] * elemsize, ldn_dst[3] * elemsize,
+				    async_data);
+	}
+	else if(ndim == 3)
+	{
+		return starpu_interface_copy3d(src, src_offset, src_node,
+				    dst, dst_offset, dst_node,
+				    nn[0] * elemsize,
+				    nn[1], ldn_src[1] * elemsize, ldn_dst[1] * elemsize,
+				    nn[2], ldn_src[2] * elemsize, ldn_dst[2] * elemsize,
+				    async_data);
+	}
+	else if(ndim == 2)
+	{
+		return starpu_interface_copy2d(src, src_offset, src_node,
+				    dst, dst_offset, dst_node,
+				    nn[0] * elemsize,
+				    nn[1], ldn_src[1] * elemsize, ldn_dst[1] * elemsize,
+				    async_data);
+	}
+	else if (ndim == 1)
+	{
+		return starpu_interface_copy(src, src_offset, src_node,
+					     dst, dst_offset, dst_node,
+					     nn[0] * elemsize,
+					     async_data);
+	}
+	else if (ndim == 0)
+	{
+		return starpu_interface_copy(src, 0, src_node,
+					     dst, 0, dst_node,
+					     elemsize,
+					     async_data);
+	}
+
+	return ret;
+}
+
 void _starpu_driver_wait_request_completion(struct _starpu_async_channel *async_channel)
 {
 #ifdef STARPU_SIMGRID
 	_starpu_simgrid_wait_transfer_event(&async_channel->event);
 #else /* !SIMGRID */
-	struct _starpu_node_ops *node_ops = async_channel->node_ops;
+	const struct _starpu_node_ops *node_ops = async_channel->node_ops;
 	if (node_ops && node_ops->wait_request_completion != NULL)
 	{
 		node_ops->wait_request_completion(async_channel);
@@ -470,7 +762,7 @@ unsigned _starpu_driver_test_request_completion(struct _starpu_async_channel *as
 #ifdef STARPU_SIMGRID
 	return _starpu_simgrid_test_transfer_event(&async_channel->event);
 #else /* !SIMGRID */
-	struct _starpu_node_ops *node_ops = async_channel->node_ops;
+	const struct _starpu_node_ops *node_ops = async_channel->node_ops;
 	if (node_ops && node_ops->test_request_completion != NULL)
 	{
 		return node_ops->test_request_completion(async_channel);
