@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,6 +15,9 @@
  */
 
 #include <starpu.h>
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
 
 /*
  * BCSR : blocked CSR, we use blocks of size (r x c)
@@ -27,9 +30,8 @@ static const struct starpu_data_copy_methods bcsr_copy_data_methods_s =
 	.any_to_any = copy_any_to_any,
 };
 
-static void register_bcsr_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_bcsr_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static void *bcsr_to_pointer(void *data_interface, unsigned node);
-static int bcsr_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static starpu_ssize_t allocate_bcsr_buffer_on_node(void *data_interface, unsigned dst_node);
 static void free_bcsr_buffer_on_node(void *data_interface, unsigned node);
 static size_t bcsr_interface_get_size(starpu_data_handle_t handle);
@@ -37,6 +39,7 @@ static int bcsr_compare(void *data_interface_a, void *data_interface_b);
 static uint32_t footprint_bcsr_interface_crc32(starpu_data_handle_t handle);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 static int pack_data(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 
 struct starpu_data_interface_ops starpu_interface_bcsr_ops =
@@ -52,10 +55,13 @@ struct starpu_data_interface_ops starpu_interface_bcsr_ops =
 	.compare = bcsr_compare,
 	.describe = describe,
 	.to_pointer = bcsr_to_pointer,
-	.pointer_is_inside = bcsr_pointer_is_inside,
 	.name = "STARPU_BCSR_INTERFACE",
 	.pack_data = pack_data,
-	.unpack_data = unpack_data
+	.peek_data = peek_data,
+	.unpack_data = unpack_data,
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL
 };
 
 static void *bcsr_to_pointer(void *data_interface, unsigned node)
@@ -66,24 +72,20 @@ static void *bcsr_to_pointer(void *data_interface, unsigned node)
 	return (void*) bcsr_interface->nzval;
 }
 
-static int bcsr_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
-{
-	(void) node;
-	struct starpu_bcsr_interface *bcsr_interface = data_interface;
-
-	return ((char*) ptr >= (char*) bcsr_interface->nzval &&
-		(char*) ptr < (char*) bcsr_interface->nzval + bcsr_interface->nnz*bcsr_interface->r*bcsr_interface->c*bcsr_interface->elemsize)
-	    || ((char*) ptr >= (char*) bcsr_interface->colind &&
-		(char*) ptr < (char*) bcsr_interface->colind + bcsr_interface->nnz*sizeof(uint32_t))
-	    || ((char*) ptr >= (char*) bcsr_interface->rowptr &&
-		(char*) ptr < (char*) bcsr_interface->rowptr + (bcsr_interface->nrow+1)*sizeof(uint32_t));
-}
-
-static void register_bcsr_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_bcsr_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_bcsr_interface *bcsr_interface = (struct starpu_bcsr_interface *) data_interface;
 
-	unsigned node;
+	int node;
+	uint32_t *ram_colind = NULL;
+	uint32_t *ram_rowptr = NULL;
+
+	if (home_node >= 0 && starpu_node_get_kind(home_node) == STARPU_CPU_RAM)
+	{
+		ram_colind = bcsr_interface->colind;
+		ram_rowptr = bcsr_interface->rowptr;
+	}
+
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_bcsr_interface *local_interface = (struct starpu_bcsr_interface *)
@@ -102,6 +104,8 @@ static void register_bcsr_handle(starpu_data_handle_t handle, unsigned home_node
 			local_interface->rowptr = NULL;
 		}
 
+		local_interface->ram_colind = ram_colind;
+		local_interface->ram_rowptr = ram_rowptr;
 		local_interface->id = bcsr_interface->id;
 		local_interface->nnz = bcsr_interface->nnz;
 		local_interface->nrow = bcsr_interface->nrow;
@@ -113,9 +117,9 @@ static void register_bcsr_handle(starpu_data_handle_t handle, unsigned home_node
 }
 
 void starpu_bcsr_data_register(starpu_data_handle_t *handleptr, int home_node,
-		uint32_t nnz, uint32_t nrow, uintptr_t nzval, uint32_t *colind,
-		uint32_t *rowptr, uint32_t firstentry,
-		uint32_t r, uint32_t c, size_t elemsize)
+			       uint32_t nnz, uint32_t nrow, uintptr_t nzval, uint32_t *colind,
+			       uint32_t *rowptr, uint32_t firstentry,
+			       uint32_t r, uint32_t c, size_t elemsize)
 {
 	struct starpu_bcsr_interface bcsr_interface =
 	{
@@ -167,7 +171,7 @@ static int bcsr_compare(void *data_interface_a, void *data_interface_b)
 	struct starpu_bcsr_interface *bcsr_a = (struct starpu_bcsr_interface *) data_interface_a;
 	struct starpu_bcsr_interface *bcsr_b = (struct starpu_bcsr_interface *) data_interface_b;
 
-	/* Two matricess are considered compatible if they have the same size */
+	/* Two matrices are considered compatible if they have the same size */
 	return (bcsr_a->nnz == bcsr_b->nnz)
 		&& (bcsr_a->nrow == bcsr_b->nrow)
 		&& (bcsr_a->r == bcsr_b->r)
@@ -384,9 +388,12 @@ static void free_bcsr_buffer_on_node(void *data_interface, unsigned node)
 	if (nnz)
 	{
 		starpu_free_on_node(node, bcsr_interface->nzval, nnz*r*c*elemsize);
+		bcsr_interface->nzval = 0;
 		starpu_free_on_node(node, (uintptr_t) bcsr_interface->colind, nnz*sizeof(uint32_t));
+		bcsr_interface->colind = NULL;
 	}
 	starpu_free_on_node(node, (uintptr_t) bcsr_interface->rowptr, (nrow+1)*sizeof(uint32_t));
+	bcsr_interface->rowptr = NULL;
 }
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data)
@@ -461,7 +468,7 @@ static int pack_data(starpu_data_handle_t handle, unsigned node, void **ptr, sta
 	return 0;
 }
 
-static int unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int peek_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -479,6 +486,12 @@ static int unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, si
 	}
 	memcpy((void*)bcsr->nzval, tmp, bcsr->r * bcsr->c * bcsr->nnz * bcsr->elemsize);
 
+	return 0;
+}
+
+static int unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	peek_data(handle, node, ptr, count);
 	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
 
 	return 0;
