@@ -38,7 +38,8 @@
 #include <nvml.h>
 #endif
 #ifdef STARPU_USE_CUDA
-#include <cublas.h>
+#include <starpu_cublas_v2.h>
+#include <starpu_cusolver.h>
 #endif
 #include <datawizard/memory_manager.h>
 #include <datawizard/memory_nodes.h>
@@ -58,7 +59,7 @@
 
 #ifdef STARPU_USE_CUDA
 #if CUDART_VERSION >= 5000
-/* Avoid letting our streams spuriously synchonize with the NULL stream */
+/* Avoid letting our streams spuriously synchronize with the NULL stream */
 #define starpu_cudaStreamCreate(stream) cudaStreamCreateWithFlags(stream, cudaStreamNonBlocking)
 #else
 #define starpu_cudaStreamCreate(stream) cudaStreamCreate(stream)
@@ -239,6 +240,49 @@ void _starpu_cuda_discover_devices(struct _starpu_machine_config *config)
 #endif
 }
 
+#ifdef STARPU_HAVE_LIBNVIDIA_ML
+static int _starpu_cuda_direct_link(unsigned devid1, unsigned devid2)
+{
+	unsigned i;
+	struct cudaDeviceProp props_dev1;
+	struct cudaDeviceProp props_dev2;
+	cudaError_t cures;
+
+	cures = cudaGetDeviceProperties(&props_dev1, devid1);
+	if (cures != cudaSuccess)
+		return 0;
+	cures = cudaGetDeviceProperties(&props_dev2, devid2);
+	if (cures != cudaSuccess)
+		return 0;
+
+	nvmlDevice_t nvml_dev1 = _starpu_cuda_get_nvmldev(&props_dev1);
+
+	if (!nvml_dev1)
+		return 0;
+
+	for (i = 0; i < NVML_NVLINK_MAX_LINKS; i++) {
+		nvmlEnableState_t active;
+		nvmlReturn_t ret;
+		ret = nvmlDeviceGetNvLinkState(nvml_dev1, i, &active);
+		if (ret == NVML_ERROR_NOT_SUPPORTED)
+			continue;
+		if (active != NVML_FEATURE_ENABLED)
+			continue;
+
+		nvmlPciInfo_t pci;
+		nvmlDeviceGetNvLinkRemotePciInfo(nvml_dev1, i, &pci);
+		if ((int) pci.domain == props_dev2.pciDomainID &&
+		    (int) pci.bus == props_dev2.pciBusID &&
+		    (int) pci.device == props_dev2.pciDeviceID)
+			/* We have a direct NVLink! */
+			return 1;
+	}
+
+	/* No direct NVLink found */
+	return 0;
+}
+#endif
+
 static void _starpu_initialize_workers_cuda_gpuid(struct _starpu_machine_config *config)
 {
 	struct _starpu_machine_topology *topology = &config->topology;
@@ -376,7 +420,7 @@ void _starpu_init_cuda_config(struct _starpu_machine_topology *topology, struct
 			}
 			else
 			{
-				_STARPU_DEBUG("Warning: could not find location of CUDA%u, do you have the hwloc CUDA plugin installed?\n", devid);
+				_STARPU_DISP("Warning: could not find location of CUDA%u, do you have the hwloc CUDA plugin installed?\n", devid);
 			}
 		}
 #endif
@@ -482,11 +526,24 @@ void _starpu_cuda_init_worker_memory(struct _starpu_machine_config *config, int
 				if (workerarg2->arch == STARPU_CUDA_WORKER)
 				{
 					unsigned memory_node2 = starpu_worker_get_memory_node(worker2);
-					_starpu_cuda_bus_ids[devid2+STARPU_MAXNUMANODES][devid+STARPU_MAXNUMANODES] = _starpu_register_bus(memory_node2, memory_node);
-					_starpu_cuda_bus_ids[devid+STARPU_MAXNUMANODES][devid2+STARPU_MAXNUMANODES] = _starpu_register_bus(memory_node, memory_node2);
+					int bus21 = _starpu_register_bus(memory_node2, memory_node);
+					int bus12 = _starpu_register_bus(memory_node, memory_node2);
+					if (bus21 < 0 || bus12 < 0)
+						/* Already registered because of e.g. several workers per CUDA */
+						continue;
+					_starpu_cuda_bus_ids[devid2+STARPU_MAXNUMANODES][devid+STARPU_MAXNUMANODES] = bus21;
+					_starpu_cuda_bus_ids[devid+STARPU_MAXNUMANODES][devid2+STARPU_MAXNUMANODES] = bus12;
 #ifndef STARPU_SIMGRID
-#if HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX
+#ifdef STARPU_HAVE_LIBNVIDIA_ML
+					if (_starpu_cuda_direct_link(devid, devid2))
 					{
+						starpu_bus_set_ngpus(bus21, 1);
+						starpu_bus_set_ngpus(bus12, 1);
+					}
+					else
+#endif
+					{
+#if HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX
 						hwloc_obj_t obj, obj2, ancestor;
 						obj = hwloc_cuda_get_device_osdev_by_index(config->topology.hwtopology, devid);
 						obj2 = hwloc_cuda_get_device_osdev_by_index(config->topology.hwtopology, devid2);
@@ -501,11 +558,11 @@ void _starpu_cuda_init_worker_memory(struct _starpu_machine_config *config, int
 								_STARPU_DEBUG("CUDA%u and CUDA%u are linked through %s, along %u GPUs\n", devid, devid2, name, data->ngpus);
 							}
 #endif
-							starpu_bus_set_ngpus(_starpu_cuda_bus_ids[devid2+STARPU_MAXNUMANODES][devid+STARPU_MAXNUMANODES], data->ngpus);
-							starpu_bus_set_ngpus(_starpu_cuda_bus_ids[devid+STARPU_MAXNUMANODES][devid2+STARPU_MAXNUMANODES], data->ngpus);
+							starpu_bus_set_ngpus(bus21, data->ngpus);
+							starpu_bus_set_ngpus(bus12, data->ngpus);
 						}
-					}
 #endif
+					}
 #endif
 				}
 			}
@@ -543,7 +600,7 @@ void starpu_cuda_set_device(unsigned devid STARPU_ATTRIBUTE_UNUSED)
 #elif !defined(HAVE_CUDA_GL_INTEROP_H)
 	if (conf->n_cuda_opengl_interoperability)
 	{
-		_STARPU_MSG("OpenGL interoperability was requested, but cuda_gl_interop.h could not be compiled, please make sure that OpenGL headers were available before ./configure run.");
+		_STARPU_MSG("OpenGL interoperability was requested, but cuda_gl_interop.h could not be compiled, please make sure that OpenGL headers were available before ./configure run.\n");
 		STARPU_ABORT();
 	}
 #else
@@ -1863,7 +1920,7 @@ static void start_job_on_cuda(struct _starpu_job *j, struct _starpu_worker *work
 	struct starpu_task *task = j->task;
 
 	int profiling = starpu_profiling_status_get();
-#ifdef STARPU_PROF_TOOL
+#if !defined(STARPU_SIMGRID) && defined(STARPU_PROF_TOOL)
 	struct starpu_prof_tool_info pi;
 #endif
 
@@ -1891,7 +1948,7 @@ static void start_job_on_cuda(struct _starpu_job *j, struct _starpu_worker *work
 
 	if (_starpu_get_disable_kernels() <= 0)
 	{
-		_STARPU_TRACE_START_EXECUTING();
+		_STARPU_TRACE_START_EXECUTING(j);
 #ifdef STARPU_SIMGRID
 		int async = task->cl->cuda_flags[j->nimpl] & STARPU_CUDA_ASYNC;
 		unsigned workerid = worker->workerid;
@@ -1934,7 +1991,7 @@ static void start_job_on_cuda(struct _starpu_job *j, struct _starpu_worker *work
 #endif
 
 #endif
-		_STARPU_TRACE_END_EXECUTING();
+		_STARPU_TRACE_END_EXECUTING(j);
 	}
 }
 
@@ -1978,18 +2035,6 @@ static void execute_job_on_cuda(struct starpu_task *task, struct _starpu_worker
 			cudaError_t cures = cudaEventRecord(task_events[workerid][pipeline_idx], starpu_cuda_get_local_stream());
 			if (STARPU_UNLIKELY(cures))
 				STARPU_CUDA_REPORT_ERROR(cures);
-#endif
-#ifdef STARPU_USE_FXT
-			if (fut_active)
-			{
-				int k;
-				for (k = 0; k < (int) worker->set->nworkers; k++)
-					if (worker->set->workers[k].ntasks == worker->set->workers[k].pipeline_length)
-						break;
-				if (k == (int) worker->set->nworkers)
-					/* Everybody busy */
-					_STARPU_TRACE_START_EXECUTING();
-			}
 #endif
 		}
 	}
@@ -2184,18 +2229,6 @@ int _starpu_cuda_driver_run_once(struct _starpu_worker *worker)
 					/* Data for next task didn't have time to finish transferring :/ */
 					_STARPU_TRACE_WORKER_START_FETCH_INPUT(NULL, workerid);
 			}
-#ifdef STARPU_USE_FXT
-			if (fut_active)
-			{
-				int k;
-				for (k = 0; k < (int) worker_set->nworkers; k++)
-					if (worker_set->workers[k].ntasks)
-						break;
-				if (k == (int) worker_set->nworkers)
-					/* Everybody busy */
-					_STARPU_TRACE_END_EXECUTING()
-			}
-#endif
 			_STARPU_TRACE_START_PROGRESS(memnode);
 #ifdef STARPU_PROF_TOOL
             pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_start_transfer, worker->workerid, worker->workerid, starpu_prof_tool_driver_gpu, memnode, NULL);
@@ -2357,6 +2390,51 @@ void starpu_cuda_report_error(const char *func, const char *file, int line, cuda
 	const char *errormsg = cudaGetErrorString(status);
 	_STARPU_ERROR("oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
 }
+
+#ifdef STARPU_HAVE_LIBCUSOLVER
+void starpu_cusolver_report_error(const char *func, const char *file, int line, cusolverStatus_t status)
+{
+#define REPORT(error) case error: errormsg = #error; break;
+	char *errormsg;
+	switch (status)
+	{
+		REPORT(CUSOLVER_STATUS_SUCCESS);
+		REPORT(CUSOLVER_STATUS_NOT_INITIALIZED);
+		REPORT(CUSOLVER_STATUS_ALLOC_FAILED);
+		REPORT(CUSOLVER_STATUS_INVALID_VALUE);
+		REPORT(CUSOLVER_STATUS_ARCH_MISMATCH);
+		REPORT(CUSOLVER_STATUS_MAPPING_ERROR);
+		REPORT(CUSOLVER_STATUS_EXECUTION_FAILED);
+		REPORT(CUSOLVER_STATUS_INTERNAL_ERROR);
+		REPORT(CUSOLVER_STATUS_MATRIX_TYPE_NOT_SUPPORTED);
+		REPORT(CUSOLVER_STATUS_NOT_SUPPORTED);
+		REPORT(CUSOLVER_STATUS_ZERO_PIVOT);
+		REPORT(CUSOLVER_STATUS_INVALID_LICENSE);
+
+#if CUSOLVER_VER_MAJOR >= 11
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_NOT_INITIALIZED);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_PREC);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_REFINE);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_MAXITER);
+		REPORT(CUSOLVER_STATUS_IRS_INTERNAL_ERROR);
+		REPORT(CUSOLVER_STATUS_IRS_NOT_SUPPORTED);
+		REPORT(CUSOLVER_STATUS_IRS_OUT_OF_RANGE);
+		REPORT(CUSOLVER_STATUS_IRS_NRHS_NOT_SUPPORTED_FOR_REFINE_GMRES);
+		REPORT(CUSOLVER_STATUS_IRS_INFOS_NOT_INITIALIZED);
+		REPORT(CUSOLVER_STATUS_IRS_INFOS_NOT_DESTROYED);
+		REPORT(CUSOLVER_STATUS_IRS_MATRIX_SINGULAR);
+		REPORT(CUSOLVER_STATUS_INVALID_WORKSPACE);
+#endif
+		default:
+			errormsg = "unknown error";
+			break;
+	}
+	_STARPU_MSG("oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
+	STARPU_ABORT();
+}
+#endif
+
 #endif /* STARPU_USE_CUDA */
 
 int _starpu_cuda_run_from_worker(struct _starpu_worker *worker)
