@@ -56,7 +56,7 @@
 #endif
 
 #if CUDART_VERSION >= 5000
-/* Avoid letting our streams spuriously synchonize with the NULL stream */
+/* Avoid letting our streams spuriously synchronize with the NULL stream */
 #define starpu_cudaStreamCreate(stream) cudaStreamCreateWithFlags(stream, cudaStreamNonBlocking)
 #else
 #define starpu_cudaStreamCreate(stream) cudaStreamCreate(stream)
@@ -1078,9 +1078,9 @@ static void start_job_on_cuda(struct _starpu_job *j, struct _starpu_worker *work
 
 	if (_starpu_get_disable_kernels() <= 0)
 	{
-		_STARPU_TRACE_START_EXECUTING();
+		_STARPU_TRACE_START_EXECUTING(j);
 		func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
-		_STARPU_TRACE_END_EXECUTING();
+		_STARPU_TRACE_END_EXECUTING(j);
 	}
 }
 
@@ -1107,9 +1107,6 @@ static void execute_job_on_cuda(struct starpu_task *task, struct _starpu_worker
 		cudaError_t cures = cudaEventRecord(task_events[workerid], starpu_cuda_get_local_stream());
 		if (STARPU_UNLIKELY(cures))
 			STARPU_CUDA_REPORT_ERROR(cures);
-#ifdef STARPU_USE_FXT
-		_STARPU_TRACE_START_EXECUTING();
-#endif
 	}
 	else
 	/* Synchronous execution */
@@ -1205,9 +1202,6 @@ int _starpu_cuda_driver_run_once(struct _starpu_worker *worker)
 			_STARPU_TRACE_END_PROGRESS(memnode);
 			/* Asynchronous task completed! */
 			finish_job_on_cuda(_starpu_get_job_associated_to_task(task), worker);
-#ifdef STARPU_USE_FXT
-			_STARPU_TRACE_END_EXECUTING()
-#endif
 			_STARPU_TRACE_START_PROGRESS(memnode);
 		}
 		if (worker->ntasks < 1)
@@ -1328,6 +1322,50 @@ void starpu_cuda_report_error(const char *func, const char *file, int line, cuda
 	_STARPU_ERROR("oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
 }
 
+#ifdef STARPU_HAVE_LIBCUSOLVER
+void starpu_cusolver_report_error(const char *func, const char *file, int line, cusolverStatus_t status)
+{
+#define REPORT(error) case error: errormsg = #error; break;
+	char *errormsg;
+	switch (status)
+	{
+		REPORT(CUSOLVER_STATUS_SUCCESS);
+		REPORT(CUSOLVER_STATUS_NOT_INITIALIZED);
+		REPORT(CUSOLVER_STATUS_ALLOC_FAILED);
+		REPORT(CUSOLVER_STATUS_INVALID_VALUE);
+		REPORT(CUSOLVER_STATUS_ARCH_MISMATCH);
+		REPORT(CUSOLVER_STATUS_MAPPING_ERROR);
+		REPORT(CUSOLVER_STATUS_EXECUTION_FAILED);
+		REPORT(CUSOLVER_STATUS_INTERNAL_ERROR);
+		REPORT(CUSOLVER_STATUS_MATRIX_TYPE_NOT_SUPPORTED);
+		REPORT(CUSOLVER_STATUS_NOT_SUPPORTED);
+		REPORT(CUSOLVER_STATUS_ZERO_PIVOT);
+		REPORT(CUSOLVER_STATUS_INVALID_LICENSE);
+
+#if CUSOLVER_VER_MAJOR >= 11
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_NOT_INITIALIZED);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_PREC);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_REFINE);
+		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_MAXITER);
+		REPORT(CUSOLVER_STATUS_IRS_INTERNAL_ERROR);
+		REPORT(CUSOLVER_STATUS_IRS_NOT_SUPPORTED);
+		REPORT(CUSOLVER_STATUS_IRS_OUT_OF_RANGE);
+		REPORT(CUSOLVER_STATUS_IRS_NRHS_NOT_SUPPORTED_FOR_REFINE_GMRES);
+		REPORT(CUSOLVER_STATUS_IRS_INFOS_NOT_INITIALIZED);
+		REPORT(CUSOLVER_STATUS_IRS_INFOS_NOT_DESTROYED);
+		REPORT(CUSOLVER_STATUS_IRS_MATRIX_SINGULAR);
+		REPORT(CUSOLVER_STATUS_INVALID_WORKSPACE);
+#endif
+		default:
+			errormsg = "unknown error";
+			break;
+	}
+	_STARPU_MSG("oops in %s (%s:%d)... %d: %s \n", func, file, line, status, errormsg);
+	STARPU_ABORT();
+}
+#endif
+
 int _starpu_cuda_run_from_worker(struct _starpu_worker *worker)
 {
 	/* Let's go ! */
