@@ -21,6 +21,33 @@
 #include <drivers/disk/driver_disk.h>
 #include <drivers/cpu/driver_cpu.h>
 #include <datawizard/coherency.h>
+#include <datawizard/memory_nodes.h>
+
+static struct _starpu_memory_driver_info memory_driver_info =
+{
+	.name_upper = "Disk",
+	.worker_archtype = (enum starpu_worker_archtype) -1,
+	.ops = &_starpu_driver_disk_node_ops,
+};
+
+void _starpu_disk_preinit(void)
+{
+	_starpu_memory_driver_info_register(STARPU_DISK_RAM, &memory_driver_info);
+}
+
+uintptr_t _starpu_disk_malloc_on_node(unsigned dst_node, size_t size, int flags)
+{
+	(void) flags;
+	uintptr_t addr = 0;
+	addr = (uintptr_t) _starpu_disk_alloc(dst_node, size);
+	return addr;
+}
+
+void _starpu_disk_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
+{
+	(void) flags;
+	_starpu_disk_free(dst_node, (void *) addr , size);
+}
 
 int _starpu_disk_copy_src_to_disk(void * src, unsigned src_node, void * dst, size_t dst_offset, unsigned dst_node, size_t size, void * async_channel)
 {
@@ -45,18 +72,19 @@ int _starpu_disk_copy_disk_to_disk(void * src, size_t src_offset, unsigned src_n
 
 unsigned _starpu_disk_test_request_completion(struct _starpu_async_channel *async_channel)
 {
+	struct _starpu_disk_event *disk_event = _starpu_disk_get_event(&async_channel->event);
 	unsigned success = starpu_disk_test_request(async_channel);
-	if (async_channel->event.disk_event.ptr != NULL && success)
+	if (disk_event->ptr != NULL && success)
 	{
-		if (async_channel->event.disk_event.handle != NULL)
+		if (disk_event->handle != NULL)
 		{
 			/* read is finished, we can already unpack */
-			async_channel->event.disk_event.handle->ops->unpack_data(async_channel->event.disk_event.handle, async_channel->event.disk_event.node, async_channel->event.disk_event.ptr, async_channel->event.disk_event.size);
+			disk_event->handle->ops->unpack_data(disk_event->handle, disk_event->node, disk_event->ptr, disk_event->size);
 		}
 		else
 		{
 			/* write is finished, ptr was allocated in pack_data */
-			_starpu_free_flags_on_node(async_channel->event.disk_event.node, async_channel->event.disk_event.ptr, async_channel->event.disk_event.size, 0);
+			_starpu_free_flags_on_node(disk_event->node, disk_event->ptr, disk_event->size, 0);
 		}
 	}
 	return success;
@@ -64,18 +92,19 @@ unsigned _starpu_disk_test_request_completion(struct _starpu_async_channel *asyn
 
 void _starpu_disk_wait_request_completion(struct _starpu_async_channel *async_channel)
 {
+	struct _starpu_disk_event *disk_event = _starpu_disk_get_event(&async_channel->event);
 	starpu_disk_wait_request(async_channel);
-	if (async_channel->event.disk_event.ptr != NULL)
+	if (disk_event->ptr != NULL)
 	{
-		if (async_channel->event.disk_event.handle != NULL)
+		if (disk_event->handle != NULL)
 		{
 			/* read is finished, we can already unpack */
-			async_channel->event.disk_event.handle->ops->unpack_data(async_channel->event.disk_event.handle, async_channel->event.disk_event.node, async_channel->event.disk_event.ptr, async_channel->event.disk_event.size);
+			disk_event->handle->ops->unpack_data(disk_event->handle, disk_event->node, disk_event->ptr, disk_event->size);
 		}
 		else
 		{
 			/* write is finished, ptr was allocated in pack_data */
-			_starpu_free_flags_on_node(async_channel->event.disk_event.node, async_channel->event.disk_event.ptr, async_channel->event.disk_event.size, 0);
+			_starpu_free_flags_on_node(disk_event->node, disk_event->ptr, disk_event->size, 0);
 		}
 	}
 }
@@ -88,13 +117,14 @@ int _starpu_disk_copy_interface_from_disk_to_cpu(starpu_data_handle_t handle, vo
 
 	int ret = 0;
 	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
+	struct _starpu_disk_event *disk_event = _starpu_disk_get_event(&req->async_channel.event);
 
 	if (req && !starpu_asynchronous_copy_disabled())
 	{
 		req->async_channel.node_ops = &_starpu_driver_disk_node_ops;
-		req->async_channel.event.disk_event.requests = NULL;
-		req->async_channel.event.disk_event.ptr = NULL;
-		req->async_channel.event.disk_event.handle = NULL;
+		disk_event->requests = NULL;
+		disk_event->ptr = NULL;
+		disk_event->handle = NULL;
 	}
 	if(copy_methods->any_to_any)
 		ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, req && !starpu_asynchronous_copy_disabled()  ? &req->async_channel : NULL);
@@ -112,10 +142,10 @@ int _starpu_disk_copy_interface_from_disk_to_cpu(starpu_data_handle_t handle, vo
 		else if (ret == -EAGAIN)
 		{
 			STARPU_ASSERT(req);
-			req->async_channel.event.disk_event.ptr = ptr;
-			req->async_channel.event.disk_event.node = dst_node;
-			req->async_channel.event.disk_event.size = size;
-			req->async_channel.event.disk_event.handle = handle;
+			disk_event->ptr = ptr;
+			disk_event->node = dst_node;
+			disk_event->size = size;
+			disk_event->handle = handle;
 		}
 		STARPU_ASSERT(ret == 0 || ret == -EAGAIN);
 	}
@@ -134,10 +164,11 @@ int _starpu_disk_copy_interface_from_disk_to_disk(starpu_data_handle_t handle, v
 
 	if (req && !starpu_asynchronous_copy_disabled())
 	{
+		struct _starpu_disk_event *disk_event = _starpu_disk_get_event(&req->async_channel.event);
 		req->async_channel.node_ops = &_starpu_driver_disk_node_ops;
-		req->async_channel.event.disk_event.requests = NULL;
-		req->async_channel.event.disk_event.ptr = NULL;
-		req->async_channel.event.disk_event.handle = NULL;
+		disk_event->requests = NULL;
+		disk_event->ptr = NULL;
+		disk_event->handle = NULL;
 	}
 	ret = copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, req && !starpu_asynchronous_copy_disabled() ? &req->async_channel : NULL);
 	return ret;
@@ -151,13 +182,14 @@ int _starpu_disk_copy_interface_from_cpu_to_disk(starpu_data_handle_t handle, vo
 
 	int ret = 0;
 	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
+	struct _starpu_disk_event *disk_event = _starpu_disk_get_event(&req->async_channel.event);
 
 	if (req && !starpu_asynchronous_copy_disabled())
 	{
 		req->async_channel.node_ops = &_starpu_driver_disk_node_ops;
-		req->async_channel.event.disk_event.requests = NULL;
-		req->async_channel.event.disk_event.ptr = NULL;
-		req->async_channel.event.disk_event.handle = NULL;
+		disk_event->requests = NULL;
+		disk_event->ptr = NULL;
+		disk_event->handle = NULL;
 	}
 
 	if(copy_methods->any_to_any)
@@ -177,9 +209,9 @@ int _starpu_disk_copy_interface_from_cpu_to_disk(starpu_data_handle_t handle, vo
 		else if (ret == -EAGAIN)
 		{
 			STARPU_ASSERT(req);
-			req->async_channel.event.disk_event.ptr = ptr;
-			req->async_channel.event.disk_event.node = src_node;
-			req->async_channel.event.disk_event.size = size;
+			disk_event->ptr = ptr;
+			disk_event->node = src_node;
+			disk_event->size = size;
 		}
 		STARPU_ASSERT(ret == 0 || ret == -EAGAIN);
 	}
@@ -234,44 +266,29 @@ int _starpu_disk_is_direct_access_supported(unsigned node, unsigned handling_nod
 	}
 }
 
-uintptr_t _starpu_disk_malloc_on_node(unsigned dst_node, size_t size, int flags)
-{
-	(void) flags;
-	uintptr_t addr = 0;
-	addr = (uintptr_t) _starpu_disk_alloc(dst_node, size);
-	return addr;
-}
-
-void _starpu_disk_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
-{
-	(void) flags;
-	_starpu_disk_free(dst_node, (void *) addr , size);
-}
-
 struct _starpu_node_ops _starpu_driver_disk_node_ops =
 {
-	.copy_interface_to[STARPU_UNUSED] = NULL,
+	.name = "disk driver",
+
+	.malloc_on_node = _starpu_disk_malloc_on_node,
+	.free_on_node = _starpu_disk_free_on_node,
+
+	.is_direct_access_supported = _starpu_disk_is_direct_access_supported,
+
 	.copy_interface_to[STARPU_CPU_RAM] = _starpu_disk_copy_interface_from_disk_to_cpu,
-	.copy_interface_to[STARPU_CUDA_RAM] = NULL,
-	.copy_interface_to[STARPU_OPENCL_RAM] = NULL,
 	.copy_interface_to[STARPU_DISK_RAM] = _starpu_disk_copy_interface_from_disk_to_disk,
-	.copy_interface_to[STARPU_MIC_RAM] = NULL,
-	.copy_interface_to[STARPU_MPI_MS_RAM] = NULL,
+
+	.copy_interface_from[STARPU_CPU_RAM] = _starpu_disk_copy_interface_from_cpu_to_disk,
+	.copy_interface_from[STARPU_DISK_RAM] = _starpu_disk_copy_interface_from_disk_to_disk,
 
-	.copy_data_to[STARPU_UNUSED] = NULL,
 	.copy_data_to[STARPU_CPU_RAM] = _starpu_disk_copy_data_from_disk_to_cpu,
-	.copy_data_to[STARPU_CUDA_RAM] = NULL,
-	.copy_data_to[STARPU_OPENCL_RAM] = NULL,
 	.copy_data_to[STARPU_DISK_RAM] = _starpu_disk_copy_data_from_disk_to_disk,
-	.copy_data_to[STARPU_MIC_RAM] = NULL,
-	.copy_data_to[STARPU_MPI_MS_RAM] = NULL,
+
+	.copy_data_from[STARPU_CPU_RAM] = _starpu_disk_copy_data_from_cpu_to_disk,
+	.copy_data_from[STARPU_DISK_RAM] = _starpu_disk_copy_data_from_disk_to_disk,
 
 	/* TODO: copy2D/3D? */
 
 	.wait_request_completion = _starpu_disk_wait_request_completion,
 	.test_request_completion = _starpu_disk_test_request_completion,
-	.is_direct_access_supported = _starpu_disk_is_direct_access_supported,
-	.malloc_on_node = _starpu_disk_malloc_on_node,
-	.free_on_node = _starpu_disk_free_on_node,
-	.name = "disk driver"
 };
