@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,14 +21,15 @@
 
 #include <drivers/mp_common/sink_common.h>
 
+#pragma GCC visibility push(hidden)
+
 #ifdef STARPU_USE_MPI_MASTER_SLAVE
 
 void _starpu_mpi_sink_init(struct _starpu_mp_node *node);
-void _starpu_mpi_sink_deinit(struct _starpu_mp_node *node);
-void _starpu_mpi_sink_launch_workers(struct _starpu_mp_node *node);
 void _starpu_mpi_sink_bind_thread(const struct _starpu_mp_node *mp_node STARPU_ATTRIBUTE_UNUSED, int coreid, int * core_table, int nb_core);
-void (*_starpu_mpi_sink_lookup (const struct _starpu_mp_node * node STARPU_ATTRIBUTE_UNUSED, char* func_name))(void);
 
 #endif  /* STARPU_USE_MPI_MASTER_SLAVE */
 
+#pragma GCC visibility pop
+
 #endif	/* __DRIVER_MPI_SINK_H__ */
