@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -31,6 +31,9 @@
 #include <common/utils.h>
 #include "driver_opencl_utils.h"
 #include "driver_opencl.h"
+#ifdef STARPU_DEVEL
+#include <common/config-src-build.h>
+#endif
 
 #ifdef HAVE_CL_CL_EXT_H
 #include <CL/cl_ext.h>
@@ -38,9 +41,6 @@
 
 char *_starpu_opencl_program_dir;
 
-#define _STARPU_STRINGIFY_(x) #x
-#define _STARPU_STRINGIFY(x) _STARPU_STRINGIFY_(x)
-
 static
 int _starpu_opencl_locate_file(const char *source_file_name, char **located_file_name, char **located_dir_name)
 {
@@ -79,8 +79,8 @@ int _starpu_opencl_locate_file(const char *source_file_name, char **located_file
 
 	if (ret == EXIT_FAILURE)
 	{
-		_STARPU_CALLOC(*located_file_name, 1, strlen(_STARPU_STRINGIFY(STARPU_OPENCL_DATADIR))+1+strlen(source_file_name)+1);
-		snprintf(*located_file_name, strlen(_STARPU_STRINGIFY(STARPU_OPENCL_DATADIR))+1+strlen(source_file_name)+1, "%s/%s", _STARPU_STRINGIFY(STARPU_OPENCL_DATADIR), source_file_name);
+		_STARPU_CALLOC(*located_file_name, 1, strlen(STARPU_OPENCL_DATADIR)+1+strlen(source_file_name)+1);
+		snprintf(*located_file_name, strlen(STARPU_OPENCL_DATADIR)+1+strlen(source_file_name)+1, "%s/%s", STARPU_OPENCL_DATADIR, source_file_name);
 		_STARPU_DEBUG("Trying to locate <%s>\n", *located_file_name);
 		if (access(*located_file_name, R_OK) == 0)
 			ret = EXIT_SUCCESS;
@@ -110,8 +110,7 @@ int _starpu_opencl_locate_file(const char *source_file_name, char **located_file
 	return ret;
 }
 
-cl_int starpu_opencl_load_kernel(cl_kernel *kernel, cl_command_queue *queue, struct starpu_opencl_program *opencl_programs,
-                                 const char *kernel_name, int devid)
+cl_int starpu_opencl_load_kernel(cl_kernel *kernel, cl_command_queue *queue, struct starpu_opencl_program *opencl_programs, const char *kernel_name, int devid)
 {
 	cl_int err;
 	cl_device_id device;
@@ -183,7 +182,7 @@ char *_starpu_opencl_load_program_binary(const char *filename, size_t *len)
 	struct stat statbuf;
 	FILE        *fh;
 	char        *binary;
-        int         err;
+	int         err;
 
 	fh = fopen(filename, "r");
 	if (fh == 0)
