@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Thibaut Lambert
  *
@@ -22,7 +22,6 @@
 #include <limits.h>
 #include <core/workers.h>
 #include <core/perfmodel/perfmodel.h>
-#include <starpu_parameters.h>
 #include <core/detect_combined_workers.h>
 #include <core/sched_policy.h>
 #include <core/task.h>
@@ -67,9 +66,10 @@ static double worker_exp_end[STARPU_NMAXWORKERS];
 static double worker_exp_len[STARPU_NMAXWORKERS];
 static int ntasks[STARPU_NMAXWORKERS];
 
-
-/*!!!!!!! It doesn't work with several contexts because the combined workers are constructed
-  from the workers available to the program, and not to the context !!!!!!!!!!!!!!!!!!!!!!!
+/*!!!!!!!
+ * It doesn't work with several contexts because the combined workers
+ * are constructed from the workers available to the program, and not
+ * to the context !!!!!!!!!!!!!!!!!!!!!!!
 */
 
 static void parallel_heft_pre_exec_hook(struct starpu_task *task, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
@@ -327,7 +327,7 @@ static int _parallel_heft_push_task(struct starpu_task *task, unsigned prio, uns
 	worker_ctx = 0;
 	while(workers->has_next(workers, &it))
 	{
-                workerid = workers->get_next(workers, &it);
+		workerid = workers->get_next(workers, &it);
 
 		for (nimpl = 0; nimpl < STARPU_MAXIMPLEMENTATIONS; nimpl++)
 		{
@@ -474,6 +474,8 @@ static int _parallel_heft_push_task(struct starpu_task *task, unsigned prio, uns
 	else
 	{
 		//penality_best = local_data_penalty[best_id_ctx][nimpl_best];
+		STARPU_ASSERT(best_id_ctx != -1);
+		STARPU_ASSERT(nimpl_best != -1);
 		best_exp_end = local_exp_end[best_id_ctx][nimpl_best];
 	}
 	best_exp_start = _worker_exp_end[best];
@@ -493,10 +495,10 @@ static int parallel_heft_push_task(struct starpu_task *task)
 	if (task->priority == STARPU_MAX_PRIO)
 	{
 		ret_val = _parallel_heft_push_task(task, 1, sched_ctx_id);
-                return ret_val;
-        }
+		return ret_val;
+	}
 
-        ret_val = _parallel_heft_push_task(task, 0, sched_ctx_id);
+	ret_val = _parallel_heft_push_task(task, 0, sched_ctx_id);
 	return ret_val;
 }
 
@@ -566,14 +568,14 @@ static void initialize_parallel_heft_policy(unsigned sched_ctx_id)
 
 	starpu_sched_ctx_set_policy_data(sched_ctx_id, (void*)hd);
 
-	hd->alpha = starpu_get_env_float_default("STARPU_SCHED_ALPHA", _STARPU_SCHED_ALPHA_DEFAULT);
-	hd->beta = starpu_get_env_float_default("STARPU_SCHED_BETA", _STARPU_SCHED_BETA_DEFAULT);
+	hd->alpha = starpu_getenv_float_default("STARPU_SCHED_ALPHA", _STARPU_SCHED_ALPHA_DEFAULT);
+	hd->beta = starpu_getenv_float_default("STARPU_SCHED_BETA", _STARPU_SCHED_BETA_DEFAULT);
 #ifdef STARPU_NON_BLOCKING_DRIVERS
 	if (starpu_getenv("STARPU_SCHED_GAMMA"))
 		_STARPU_DISP("Warning: STARPU_SCHED_GAMMA was used, but --enable-blocking-drivers configuration was not set, CPU cores will not actually be sleeping\n");
 #endif
-	hd->_gamma = starpu_get_env_float_default("STARPU_SCHED_GAMMA", _STARPU_SCHED_GAMMA_DEFAULT);
-	hd->idle_power = starpu_get_env_float_default("STARPU_IDLE_POWER", 0.0);
+	hd->_gamma = starpu_getenv_float_default("STARPU_SCHED_GAMMA", _STARPU_SCHED_GAMMA_DEFAULT);
+	hd->idle_power = starpu_getenv_float_default("STARPU_IDLE_POWER", 0.0);
 
 	STARPU_PTHREAD_MUTEX_INIT(&hd->global_push_mutex, NULL);
 
@@ -602,8 +604,8 @@ struct starpu_sched_policy _starpu_sched_parallel_heft_policy =
 	.pop_task = NULL,
 	.pre_exec_hook = parallel_heft_pre_exec_hook,
 	.post_exec_hook = parallel_heft_post_exec_hook,
-	.pop_every_task = NULL,
 	.policy_name = "pheft",
 	.policy_description = "parallel HEFT",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
