@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +18,8 @@
 #include <common/config.h>
 #include <core/jobs.h>
 #include <core/task.h>
+#include <core/workers.h>
+#include <profiling/callbacks.h>
 
 struct wrapper_func_args
 {
@@ -28,7 +30,22 @@ struct wrapper_func_args
 static void wrapper_func(void *buffers[] STARPU_ATTRIBUTE_UNUSED, void *_args)
 {
 	struct wrapper_func_args *args = (struct wrapper_func_args *) _args;
+#ifdef STARPU_PROF_TOOL
+	struct starpu_prof_tool_info pi;
+#endif
+
+#ifdef STARPU_PROF_TOOL
+	int worker = starpu_worker_get_id();
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_start_gpu_exec, worker, worker, starpu_prof_tool_driver_gpu, -1, (void*)args->func);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_start_gpu_exec(&pi, NULL, NULL);
+#endif
+
 	args->func(args->arg);
+
+#ifdef STARPU_PROF_TOOL
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_gpu_exec, worker, worker, starpu_prof_tool_driver_gpu, -1, (void*)args->func);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_end_gpu_exec(&pi, NULL, NULL);
+#endif
 }
 
 /**
@@ -45,9 +62,9 @@ void starpu_execute_on_specific_workers(void (*func)(void*), void * arg, unsigne
 	{
 		.where = 0xFF,
 		.cuda_funcs = {wrapper_func},
+		.hip_funcs = {wrapper_func},
 		.cpu_funcs = {wrapper_func},
 		.opencl_funcs = {wrapper_func},
-		/* XXX we do not handle Cell .. */
 		.nbuffers = 0,
 		.name = name
 	};
@@ -106,14 +123,14 @@ void starpu_execute_on_each_worker_ex(void (*func)(void *), void *arg, uint32_t
 	unsigned nworkers = starpu_worker_get_count();
 	struct starpu_task *tasks[STARPU_NMAXWORKERS];
 
-	/* This method only work on CPU, CUDA, OPENCL */
-	STARPU_ASSERT((where & ~STARPU_CPU & ~STARPU_CUDA & ~STARPU_OPENCL) == 0);
+	STARPU_ASSERT_MSG((where & ~STARPU_CPU & ~STARPU_CUDA & ~STARPU_OPENCL & ~STARPU_HIP) == 0, "This function is implemented only on CPU, CUDA, HIP, OpenCL");
 
 	/* create a wrapper codelet */
 	struct starpu_codelet wrapper_cl =
 	{
 		.where = where,
 		.cuda_funcs = {wrapper_func},
+		.hip_funcs = {wrapper_func},
 		.cpu_funcs = {wrapper_func},
 		.opencl_funcs = {wrapper_func},
 		.nbuffers = 0,
