@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -32,6 +32,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <strings.h>
+#include <datawizard/memory_nodes.h>
 
 #define _STARPU_INITIAL_THREAD_STACKSIZE 2097152
 
@@ -42,6 +43,18 @@ starpu_pthread_key_t _starpu_omp_task_key;
 struct starpu_omp_global *_starpu_omp_global_state = NULL;
 double _starpu_omp_clock_ref = 0.0; /* clock reference for starpu_omp_get_wtick */
 
+/* Entry in the `registered_handles' hash table.  */
+struct handle_entry
+{
+	UT_hash_handle hh;
+	void *pointer;
+	starpu_data_handle_t handle;
+};
+
+
+static struct handle_entry *registered_handles;
+static struct _starpu_spinlock    registered_handles_lock;
+
 static struct starpu_omp_critical *create_omp_critical_struct(void);
 static void destroy_omp_critical_struct(struct starpu_omp_critical *critical);
 static struct starpu_omp_device *create_omp_device_struct(void);
@@ -57,7 +70,7 @@ static void wake_up_and_unlock_task(struct starpu_omp_task *task);
 static void wake_up_barrier(struct starpu_omp_region *parallel_region);
 static void starpu_omp_task_preempt(void);
 
-struct starpu_omp_thread *_starpu_omp_get_thread(void)
+struct starpu_omp_thread * _starpu_omp_get_thread(void)
 {
 	struct starpu_omp_thread *thread = STARPU_PTHREAD_GETSPECIFIC(_starpu_omp_thread_key);
 	return thread;
@@ -239,16 +252,19 @@ static struct starpu_omp_thread *get_local_thread(void)
 		thread = get_worker_thread(starpu_worker);
 
 		if (
+#ifdef STARPU_DEVEL
+#warning Why not just checking for STARPU_CPU_WORKER?
+#endif
 #ifdef STARPU_USE_CUDA
-				(starpu_worker->arch != STARPU_CUDA_WORKER)
-				&&
+		    (starpu_worker->arch != STARPU_CUDA_WORKER)
+		    &&
 #endif
 #ifdef STARPU_USE_OPENCL
-				(starpu_worker->arch != STARPU_OPENCL_WORKER)
-				&&
+		    (starpu_worker->arch != STARPU_OPENCL_WORKER)
+		    &&
 #endif
-				1
-		   )
+		    1
+		    )
 		{
 			STARPU_ASSERT(thread != NULL);
 		}
@@ -261,6 +277,11 @@ static struct starpu_omp_thread *get_local_thread(void)
 	return thread;
 }
 
+static struct starpu_omp_thread * __attribute__ ((noinline)) _get_local_thread_noinline(void)
+{
+	return get_local_thread();
+}
+
 static struct starpu_omp_critical *create_omp_critical_struct(void)
 {
 	struct starpu_omp_critical *critical;
@@ -319,7 +340,7 @@ static struct starpu_omp_region *create_omp_region_struct(struct starpu_omp_regi
 	_STARPU_CALLOC(region, 1, sizeof(*region));
 	region->parent_region = parent_region;
 	region->owner_device = owner_device;
-	starpu_omp_thread_list_init(&region->thread_list);
+	starpu_omp_thread_list_init0(&region->thread_list);
 
 	_starpu_spin_init(&region->lock);
 	_starpu_spin_init(&region->registered_handles_lock);
@@ -377,6 +398,216 @@ static void destroy_omp_thread_struct(struct starpu_omp_thread *thread)
 	starpu_omp_thread_delete(thread);
 }
 
+/* Register the mapping from PTR to HANDLE.  If PTR is already mapped to
+ * some handle, the new mapping shadows the previous one.   */
+static void register_ram_pointer(starpu_data_handle_t handle, void *ptr)
+{
+	struct handle_entry *entry;
+
+	_STARPU_MALLOC(entry, sizeof(*entry));
+
+	entry->pointer = ptr;
+	entry->handle = handle;
+
+	struct starpu_omp_task *task = _starpu_omp_get_task();
+	if (task)
+	{
+		if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
+		{
+			struct starpu_omp_region *parallel_region = task->owner_region;
+			_starpu_spin_lock(&parallel_region->registered_handles_lock);
+			HASH_ADD_PTR(parallel_region->registered_handles, pointer, entry);
+			_starpu_spin_unlock(&parallel_region->registered_handles_lock);
+		}
+		else
+		{
+			HASH_ADD_PTR(task->registered_handles, pointer, entry);
+		}
+	}
+	else
+	{
+		struct handle_entry *old_entry;
+
+		_starpu_spin_lock(&registered_handles_lock);
+		HASH_FIND_PTR(registered_handles, &ptr, old_entry);
+		if (old_entry)
+		{
+			/* Already registered this pointer, avoid undefined
+			 * behavior of duplicate in hash table */
+			_starpu_spin_unlock(&registered_handles_lock);
+			free(entry);
+		}
+		else
+		{
+			HASH_ADD_PTR(registered_handles, pointer, entry);
+			_starpu_spin_unlock(&registered_handles_lock);
+		}
+	}
+}
+
+void starpu_omp_handle_register(starpu_data_handle_t handle)
+{
+	unsigned node;
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+			continue;
+
+		void *ptr = starpu_data_handle_to_pointer(handle, node);
+		if (ptr != NULL)
+			register_ram_pointer(handle, ptr);
+	}
+}
+
+/*
+ * Stop monitoring a piece of data
+ */
+static void unregister_ram_pointer(starpu_data_handle_t handle, unsigned node)
+{
+	if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+		return;
+
+	if (handle->removed_from_context_hash)
+		return;
+	const void *ram_ptr = starpu_data_handle_to_pointer(handle, node);
+
+	if (ram_ptr != NULL)
+	{
+		/* Remove the PTR -> HANDLE mapping.  If a mapping from PTR
+		 * to another handle existed before (e.g., when using
+		 * filters), it becomes visible again.  */
+		struct handle_entry *entry;
+		struct starpu_omp_task *task = _starpu_omp_get_task();
+		if (task)
+		{
+			if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
+			{
+				struct starpu_omp_region *parallel_region = task->owner_region;
+				_starpu_spin_lock(&parallel_region->registered_handles_lock);
+				HASH_FIND_PTR(parallel_region->registered_handles, &ram_ptr, entry);
+				STARPU_ASSERT(entry != NULL);
+				HASH_DEL(registered_handles, entry);
+				_starpu_spin_unlock(&parallel_region->registered_handles_lock);
+			}
+			else
+			{
+				HASH_FIND_PTR(task->registered_handles, &ram_ptr, entry);
+				STARPU_ASSERT(entry != NULL);
+				HASH_DEL(task->registered_handles, entry);
+			}
+		}
+		else
+		{
+
+			_starpu_spin_lock(&registered_handles_lock);
+			HASH_FIND_PTR(registered_handles, &ram_ptr, entry);
+			if (entry)
+			{
+				if (entry->handle == handle)
+				{
+					HASH_DEL(registered_handles, entry);
+				}
+				else
+					/* don't free it, it's not ours */
+					entry = NULL;
+			}
+			_starpu_spin_unlock(&registered_handles_lock);
+		}
+		free(entry);
+	}
+}
+
+void starpu_omp_handle_unregister(starpu_data_handle_t handle)
+{
+	unsigned node;
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		struct _starpu_data_replicate *local = &handle->per_node[node];
+		STARPU_ASSERT(!local->refcnt);
+		if (local->allocated)
+		{
+			unregister_ram_pointer(handle, node);
+		}
+	}
+}
+
+static void unregister_region_handles(struct starpu_omp_region *region)
+{
+	_starpu_spin_lock(&region->registered_handles_lock);
+	struct handle_entry *entry=NULL, *tmp=NULL;
+	HASH_ITER(hh, (region->registered_handles), entry, tmp)
+	{
+		entry->handle->removed_from_context_hash = 1;
+		HASH_DEL(region->registered_handles, entry);
+		starpu_data_unregister(entry->handle);
+		free(entry);
+	}
+	_starpu_spin_unlock(&region->registered_handles_lock);
+}
+
+static void unregister_task_handles(struct starpu_omp_task *task)
+{
+	struct handle_entry *entry=NULL, *tmp=NULL;
+	HASH_ITER(hh, task->registered_handles, entry, tmp)
+	{
+		entry->handle->removed_from_context_hash = 1;
+		HASH_DEL(task->registered_handles, entry);
+		starpu_data_unregister(entry->handle);
+		free(entry);
+	}
+}
+
+starpu_data_handle_t starpu_omp_data_lookup(const void *ptr)
+{
+	starpu_data_handle_t result;
+
+	struct starpu_omp_task *task = _starpu_omp_get_task();
+	if (task)
+	{
+		if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
+		{
+			struct starpu_omp_region *parallel_region = task->owner_region;
+			_starpu_spin_lock(&parallel_region->registered_handles_lock);
+			{
+				struct handle_entry *entry;
+
+				HASH_FIND_PTR(parallel_region->registered_handles, &ptr, entry);
+				if(STARPU_UNLIKELY(entry == NULL))
+					result = NULL;
+				else
+					result = entry->handle;
+			}
+			_starpu_spin_unlock(&parallel_region->registered_handles_lock);
+		}
+		else
+		{
+			struct handle_entry *entry;
+
+			HASH_FIND_PTR(task->registered_handles, &ptr, entry);
+			if(STARPU_UNLIKELY(entry == NULL))
+				result = NULL;
+			else
+				result = entry->handle;
+		}
+	}
+	else
+	{
+		_starpu_spin_lock(&registered_handles_lock);
+		{
+			struct handle_entry *entry;
+
+			HASH_FIND_PTR(registered_handles, &ptr, entry);
+			if(STARPU_UNLIKELY(entry == NULL))
+				result = NULL;
+			else
+				result = entry->handle;
+		}
+		_starpu_spin_unlock(&registered_handles_lock);
+	}
+
+	return result;
+}
+
 static void starpu_omp_explicit_task_entry(struct starpu_omp_task *task)
 {
 	STARPU_ASSERT(!(task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT));
@@ -405,7 +636,7 @@ static void starpu_omp_explicit_task_entry(struct starpu_omp_task *task)
 	else
 		_STARPU_ERROR("invalid worker architecture");
 	/**/
-	_starpu_omp_unregister_task_handles(task);
+	unregister_task_handles(task);
 	_starpu_spin_lock(&task->lock);
 	task->state = starpu_omp_task_state_terminated;
 	task->transaction_pending=1;
@@ -428,7 +659,7 @@ static void starpu_omp_implicit_task_entry(struct starpu_omp_task *task)
 	starpu_omp_barrier();
 	if (thread == task->owner_region->master_thread)
 	{
-		_starpu_omp_unregister_region_handles(task->owner_region);
+		unregister_region_handles(task->owner_region);
 	}
 	task->state = starpu_omp_task_state_terminated;
 	/*
@@ -467,7 +698,11 @@ static void starpu_omp_implicit_task_exec(void *buffers[], void *cl_arg)
 	struct starpu_omp_task *task = starpu_task_get_current()->omp_task;
 	STARPU_ASSERT(task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT);
 	_starpu_omp_set_task(task);
-	struct starpu_omp_thread *thread = get_local_thread();
+
+	/* get_local_thread() inlining triggers a clobbering warning with some
+	 * versions of GCC, thus we explicitly call the noinline variant */
+	struct starpu_omp_thread *thread = _get_local_thread_noinline();
+
 	if (task->state != starpu_omp_task_state_preempted)
 	{
 		task->starpu_buffers = buffers;
@@ -598,7 +833,10 @@ static void starpu_omp_explicit_task_exec(void *buffers[], void *cl_arg)
 	STARPU_ASSERT(!(task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT));
 	_starpu_omp_set_task(task);
 
-	struct starpu_omp_thread *thread = get_local_thread();
+	/* get_local_thread() inlining triggers a clobbering warning with some
+	 * versions of GCC, thus we explicitly call the noinline variant */
+	struct starpu_omp_thread *thread = _get_local_thread_noinline();
+
 	if (task->state != starpu_omp_task_state_preempted)
 	{
 		if (thread == NULL)
@@ -765,6 +1003,12 @@ static int omp_initial_thread_setup(void)
 	initial_thread->starpu_driver.id.cpu_id = 0;
 	omp_starpu_conf.not_launched_drivers = &initial_thread->starpu_driver;
 	omp_starpu_conf.n_not_launched_drivers = 1;
+#ifdef STARPU_DEVEL
+#warning setting nhip to 0 should not be necessary
+#endif
+	omp_starpu_conf.nhip = 0;
+	omp_starpu_conf.nmpi_ms = 0;
+	omp_starpu_conf.ntcpip_ms = 0;
 	/* we are now ready to start StarPU */
 	ret = starpu_init(&omp_starpu_conf);
 	int check = _starpu_omp_environment_check();
@@ -960,7 +1204,7 @@ int starpu_omp_init(void)
 	_global_state.initial_task = create_omp_task_struct(NULL,
 			_global_state.initial_thread, _global_state.initial_region, 1);
 	_global_state.default_critical = create_omp_critical_struct();
-        _global_state.default_arbiter = starpu_arbiter_create();
+	_global_state.default_arbiter = starpu_arbiter_create();
 	_global_state.named_criticals = NULL;
 	_starpu_spin_init(&_global_state.named_criticals_lock);
 	_global_state.hash_workers = NULL;
@@ -968,10 +1212,12 @@ int starpu_omp_init(void)
 
 	_starpu_omp_environment_init();
 	_global_state.icvs.cancel_var = _starpu_omp_initial_icv_values->cancel_var;
+	_global_state.environment_valid = -EINVAL; /* in case starpu_init exits (e.g. on a slave) */
 	_global_state.environment_valid = omp_initial_region_setup();
 
 	/* init clock reference for starpu_omp_get_wtick */
 	_starpu_omp_clock_ref = starpu_timing_now();
+	_starpu_spin_init(&registered_handles_lock);
 
 	return _global_state.environment_valid;
 }
@@ -992,8 +1238,8 @@ void starpu_omp_shutdown(void)
 	_global_state.initial_device = NULL;
 	destroy_omp_critical_struct(_global_state.default_critical);
 	_global_state.default_critical = NULL;
-        starpu_arbiter_destroy(_global_state.default_arbiter);
-        _global_state.default_arbiter = NULL;
+	starpu_arbiter_destroy(_global_state.default_arbiter);
+	_global_state.default_arbiter = NULL;
 	_starpu_spin_lock(&_global_state.named_criticals_lock);
 	{
 		struct starpu_omp_critical *critical=NULL, *tmp=NULL;
@@ -1007,6 +1253,24 @@ void starpu_omp_shutdown(void)
 	STARPU_ASSERT(_global_state.named_criticals == NULL);
 	_starpu_spin_unlock(&_global_state.named_criticals_lock);
 	_starpu_spin_destroy(&_global_state.named_criticals_lock);
+	{
+		struct handle_entry *entry=NULL, *tmp=NULL;
+
+		if (registered_handles)
+		{
+			_STARPU_DISP("[warning] The application has not unregistered all data handles.\n");
+		}
+
+		_starpu_spin_destroy(&registered_handles_lock);
+
+		HASH_ITER(hh, registered_handles, entry, tmp)
+		{
+			HASH_DEL(registered_handles, entry);
+			free(entry);
+		}
+
+		registered_handles = NULL;
+		}
 	_starpu_spin_lock(&_global_state.hash_workers_lock);
 	{
 		struct starpu_omp_thread *thread=NULL, *tmp=NULL;
@@ -1644,20 +1908,20 @@ void starpu_omp_task_region(const struct starpu_omp_task_region_attr *attr)
 		{
 			generated_task->flags |= STARPU_OMP_TASK_FLAGS_UNDEFERRED;
 		}
-      // XXX taskgroup exist
-      if (!attr->nogroup_clause)
-      {
-         generated_task->task_group = generating_task->task_group;
-      }
-      generated_task->rank = -1;
+		// XXX taskgroup exist
+		if (!attr->nogroup_clause)
+		{
+			generated_task->task_group = generating_task->task_group;
+		}
+		generated_task->rank = -1;
 
-      /* XXX taskloop attributes */
-      generated_task->is_loop = attr->is_loop;
-      generated_task->nb_iterations = attr->nb_iterations;
-      generated_task->grainsize = attr->grainsize;
-      generated_task->chunk = attr->chunk;
-      generated_task->begin_i = attr->begin_i;
-      generated_task->end_i = attr->end_i;
+		/* XXX taskloop attributes */
+		generated_task->is_loop = attr->is_loop;
+		generated_task->nb_iterations = attr->nb_iterations;
+		generated_task->grainsize = attr->grainsize;
+		generated_task->chunk = attr->chunk;
+		generated_task->begin_i = attr->begin_i;
+		generated_task->end_i = attr->end_i;
 
 		/*
 		 * save pointer to the regions user function from the task region codelet
@@ -1886,10 +2150,10 @@ static inline void _starpu_omp_for_loop(struct starpu_omp_region *parallel_regio
 		schedule = parallel_region->icvs.run_sched_var;
 		chunk = parallel_region->icvs.run_sched_chunk_var;
 	}
-	STARPU_ASSERT(     schedule == starpu_omp_sched_static
-			|| schedule == starpu_omp_sched_dynamic
-			|| schedule == starpu_omp_sched_guided
-			|| schedule == starpu_omp_sched_auto);
+	STARPU_ASSERT(schedule == starpu_omp_sched_static
+		      || schedule == starpu_omp_sched_dynamic
+		      || schedule == starpu_omp_sched_guided
+		      || schedule == starpu_omp_sched_auto);
 	if (schedule == starpu_omp_sched_auto)
 	{
 		schedule = starpu_omp_sched_static;
@@ -2507,14 +2771,15 @@ void starpu_omp_atomic_fallback_inline_end(void)
 
 void starpu_omp_vector_annotate(starpu_data_handle_t handle, uint32_t slice_base)
 {
-	/* FIXME Oli: rather iterate over all nodes? */
-	int node = starpu_data_get_home_node(handle);
-	if (node < 0 || (starpu_node_get_kind(node) != STARPU_CPU_RAM))
-		node = STARPU_MAIN_RAM;
-	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *)
-		starpu_data_get_interface_on_node(handle, node);
-	assert(vector_interface->id == STARPU_VECTOR_INTERFACE_ID);
-	vector_interface->slice_base = slice_base;
+	unsigned node;
+
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *)
+			starpu_data_get_interface_on_node(handle, node);
+		assert(vector_interface->id == STARPU_VECTOR_INTERFACE_ID);
+		vector_interface->slice_base = slice_base;
+	}
 }
 
 struct starpu_arbiter *starpu_omp_get_default_arbiter(void)
