@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -31,7 +31,7 @@
 
 extern struct stats global_stats;
 
-#if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL) || defined(STARPU_USE_MIC)
+#if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL)
 static void
 create_and_submit_tasks(int where, starpu_data_handle_t handles[])
 {
@@ -52,10 +52,6 @@ create_and_submit_tasks(int where, starpu_data_handle_t handles[])
 	if (where & STARPU_OPENCL)
 		cl.opencl_funcs[0] = opencl_func;
 #endif
-#ifdef STARPU_USE_MIC
-	if (where & STARPU_MIC)
-		cl.mic_funcs[0] = mic_func;
-#endif
 
 	struct starpu_task *task = starpu_task_create();
 	task->synchronous = SYNCHRONOUS;
@@ -98,10 +94,6 @@ create_and_submit_tasks(int where, starpu_data_handle_t handles[])
 	if (where & STARPU_OPENCL)
 		cl3.opencl_funcs[0] = opencl_func;
 #endif
-#ifdef STARPU_USE_MIC
-	if (where & STARPU_MIC)
-		cl3.mic_funcs[0] = mic_func;
-#endif
 
 	struct starpu_task *task3 = starpu_task_create();
 	task3->synchronous = SYNCHRONOUS;
@@ -157,11 +149,6 @@ test_cuda(void)
 	expected_stats.cpu_to_opencl = 0;
 	expected_stats.opencl_to_cpu = 0;
 #endif /* !STARPU_USE_OPENCL */
-#ifdef STARPU_USE_MIC
-	expected_stats.mic = 0;
-	expected_stats.cpu_to_mic = 0;
-	expected_stats.mic_to_cpu = 0;
-#endif
 	expected_stats.cuda = 2;
 	expected_stats.cpu_to_cuda = 2;
 	expected_stats.cuda_to_cpu = 2;
@@ -207,11 +194,6 @@ test_opencl(void)
 	expected_stats.cpu_to_cuda = 0;
 	expected_stats.cuda_to_cpu = 0;
 #endif /* !STARPU_USE_CUDA */
-#ifdef STARPU_USE_MIC
-	expected_stats.mic = 0;
-	expected_stats.cpu_to_mic = 0;
-	expected_stats.mic_to_cpu = 0;
-#endif
 	expected_stats.opencl = 2;
 	expected_stats.cpu_to_opencl = 2;
 	expected_stats.opencl_to_cpu = 2;
@@ -220,56 +202,6 @@ test_opencl(void)
 }
 #endif /* !STARPU_USE_OPENCL */
 
-#ifdef STARPU_USE_MIC
-static int
-test_mic(void)
-{
-	int i;
-	int vector1[NX];
-	int vector2[NX];
-	starpu_data_handle_t handles[2];
-
-	for (i = 0; i < NX; i++)
-	{
-		vector1[i] = i;
-		vector2[i] = i;
-	}
-
-	starpu_multiformat_data_register(&handles[0], STARPU_MAIN_RAM, vector1, NX, &ops);
-	starpu_multiformat_data_register(&handles[1], STARPU_MAIN_RAM, vector2, NX, &ops);
-
-	memset(&global_stats, 0, sizeof(global_stats));
-	create_and_submit_tasks(STARPU_MIC, handles);
-
-	starpu_data_unregister(handles[0]);
-	starpu_data_unregister(handles[1]);
-
-#if DEBUG
-	print_stats(&global_stats);
-#endif
-
-	struct stats expected_stats;
-#ifdef STARPU_USE_CPU
-	expected_stats.cpu = 1;
-#endif /* !STARPU_USE_CPU */
-#ifdef STARPU_USE_OPENCL
-	expected_stats.opencl = 0;
-	expected_stats.cpu_to_opencl = 0;
-	expected_stats.opencl_to_cpu = 0;
-#endif /* !STARPU_USE_OPENCL */
-#ifdef STARPU_USE_CUDA
-	expected_stats.cuda = 0;
-	expected_stats.cpu_to_cuda = 0;
-	expected_stats.cuda_to_cpu = 0;
-#endif
-	expected_stats.mic = 2;
-	expected_stats.cpu_to_mic = 2;
-	expected_stats.mic_to_cpu = 2;
-
-	return compare_stats(&expected_stats, &global_stats);
-}
-#endif /* !STARPU_USE_CUDA */
-
 int
 main(int argc, char **argv)
 {
@@ -278,9 +210,12 @@ main(int argc, char **argv)
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
 
+#if defined(STARPU_USE_CUDA0) || defined(STARPU_USE_CUDA1)
+	conf.ncuda = 0;
+#else
 	conf.ncuda = 2;
+#endif
 	conf.nopencl = 1;
-	conf.nmic = 1;
 
 	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
@@ -294,7 +229,6 @@ main(int argc, char **argv)
 	}
 	unsigned int ncuda = starpu_cuda_worker_get_count();
 	unsigned int nopencl = starpu_opencl_worker_get_count();
-	unsigned int nmic = starpu_mic_worker_get_count();
 
 #ifdef STARPU_USE_OPENCL
 	if (nopencl > 0 && test_opencl() != 0)
@@ -310,17 +244,10 @@ main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 #endif
-#ifdef STARPU_USE_MIC
-	if (nmic > 0 && test_mic() != 0)
-	{
-		FPRINTF(stderr, "MIC FAILED \n");
-		return EXIT_FAILURE;
-	}
-#endif
 
 	starpu_shutdown();
 
-	if (ncuda == 0 && nopencl == 0 && nmic == 0)
+	if (ncuda == 0 && nopencl == 0)
 		return STARPU_TEST_SKIPPED;
 	else
 		return EXIT_SUCCESS;
