@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -48,8 +48,13 @@ static uint64_t get_total_memory_size(void)
 {
 	uint64_t size;
 	hwloc_topology_t hwtopology;
-	hwloc_topology_init(&hwtopology);
-	hwloc_topology_load(hwtopology);
+	int err;
+
+	err = hwloc_topology_init(&hwtopology);
+	STARPU_ASSERT_MSG(err == 0, "Could not initialize Hwloc topology (%s)\n", strerror(errno));
+	err = hwloc_topology_load(hwtopology);
+	STARPU_ASSERT_MSG(err == 0, "Could not load Hwloc topology (%s)\n", strerror(errno));
+
 	hwloc_obj_t root = hwloc_get_root_obj(hwtopology);
 #if HWLOC_API_VERSION >= 0x00020000
 	size = root->total_memory;
@@ -66,9 +71,9 @@ void dummy_func(void *descr[], void *_args)
 }
 
 static unsigned int i = 0;
-void f(void *arg)
+void func(void *arg)
 {
-	printf("%u\n", ++i);
+       printf("%u\n", ++i);
 }
 
 static struct starpu_codelet dummy_cl =
@@ -150,7 +155,7 @@ int main(int argc, char **argv)
 		task->handles[0] = handle_array[taskid%mb];
 		task->handles[1] = handle_array[(taskid+1)%mb];
 		task->handles[2] = handle_array[(taskid+2)%mb];
-		task->callback_func = f;
+		task->callback_func = func;
 		task->callback_arg = NULL;
 
 		ret = starpu_task_submit(task);
@@ -160,7 +165,7 @@ int main(int argc, char **argv)
 
 	for (j = 0; j < mb; j++)
 	{
-		if ( j%20 == 0 )
+		if (j%20 == 0)
 			starpu_data_unregister_submit(handle_array[j]);
 	}
 
@@ -169,7 +174,7 @@ int main(int argc, char **argv)
 
 	for (j = 0; j < mb; j++)
 	{
-		if ( j%20 != 0 )
+		if (j%20 != 0)
 			starpu_data_unregister(handle_array[j]);
 		free(host_ptr_array[j]);
 	}
