@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2013                                Inria
- * Copyright (C) 2010-2014,2016                           Université de Bordeaux
- * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,8 +38,10 @@ float *buffer[NBUFFERS_DEF];
 
 starpu_data_handle_t v_handle[NBUFFERS_DEF];
 
-void dummy_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void dummy_codelet(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
 static struct starpu_codelet cl =
@@ -77,8 +77,9 @@ static starpu_pthread_cond_t cond = STARPU_PTHREAD_COND_INITIALIZER;
 static unsigned n_synced_buffers;
 
 static
-void callback_sync_data(void *arg STARPU_ATTRIBUTE_UNUSED)
+void callback_sync_data(void *arg)
 {
+	(void)arg;
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 
 	n_synced_buffers++;
@@ -158,7 +159,7 @@ int main(int argc, char **argv)
 	for (b = 0; b < nbuffers; b++)
 	{
 		starpu_data_unregister(v_handle[b]);
-		starpu_free(buffer[b]);
+		starpu_free_noflag(buffer[b], vectorsize);
 	}
 
 	starpu_shutdown();
