@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -51,9 +51,10 @@ int main(void)
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
 	conf.precedence_over_environment_variables = 1;
+	starpu_conf_noworker(&conf);
 	conf.ncpus = 1;
-	conf.nopencl = 0;
-	conf.ncuda = 0;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
 
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
@@ -72,7 +73,7 @@ int main(void)
 	struct starpu_task *task_specific = starpu_task_create();
 	task_specific->cl = &gpu_only_cl;
 	task_specific->execute_on_a_specific_worker = 1;
-	task_specific->workerid = 0;
+	task_specific->workerid = starpu_worker_get_by_type(STARPU_CPU_WORKER, 0);
 
 	/* Only a CUDA device could execute that task ! */
 	ret = starpu_task_submit(task_specific);
