@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,28 +24,13 @@
 
 #define NITER	10
 
-void dummy_func(void *descr[], void *arg)
-{
-	(void)descr;
-	(void)arg;
-}
-
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {dummy_func},
-	.cuda_funcs = {dummy_func},
-        .opencl_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.nbuffers = 0
-};
-
 static int create_dummy_task(starpu_tag_t tag)
 {
 	struct starpu_task *task = starpu_task_create();
 
 	task->use_tag = 1;
 	task->tag_id = tag;
-	task->cl = &dummy_codelet;
+	task->cl = &starpu_codelet_nop;
 
 	int ret = starpu_task_submit(task);
 	return ret;
