@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,6 +14,7 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+#define BUILDING_STARPU
 #include <starpu.h>
 #include "core/workers.h"
 #include "../helper.h"
@@ -44,21 +45,6 @@ static int can_never_execute(unsigned workerid,
 	return 0;
 }
 
-void fake(void *buffers[], void *args)
-{
-	(void) buffers;
-	(void) args;
-}
-
-static struct starpu_codelet cl =
-{
-	.cpu_funcs    = { fake},
-	.cuda_funcs   = { fake},
-	.opencl_funcs = { fake},
-	.cpu_funcs_name = { "fake"},
-	.nbuffers     = 0
-};
-
 int main(int argc, char **argv)
 {
 	int ret;
@@ -68,11 +54,11 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 
 	task = starpu_task_create();
-	task->cl = &cl;
+	task->cl = &starpu_codelet_nop;
 	task->destroy = 0;
 	task->sched_ctx = 0;
 
-	cl.can_execute = NULL;
+	starpu_codelet_nop.can_execute = NULL;
 	ret = _starpu_worker_exists(task);
 	if (!ret)
 	{
@@ -80,7 +66,7 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	cl.can_execute = can_always_execute;
+	starpu_codelet_nop.can_execute = can_always_execute;
 	ret = _starpu_worker_exists(task);
 	if (!ret)
 	{
@@ -88,7 +74,7 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	cl.can_execute = can_never_execute;
+	starpu_codelet_nop.can_execute = can_never_execute;
 	ret = _starpu_worker_exists(task);
 	if (ret)
 	{
