@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,22 +24,6 @@
  * Test that starpu_tag_get_task returns the proper task
  */
 
-void dummy_func(void *descr[], void *arg)
-{
-	(void)descr;
-	(void)arg;
-}
-
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {dummy_func},
-	.cuda_funcs = {dummy_func},
-	.opencl_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.model = NULL,
-	.nbuffers = 0
-};
-
 static void callback(void *tag)
 {
 	fflush(stderr);
@@ -62,7 +46,7 @@ int main(int argc, char **argv)
 	task = starpu_task_create();
 	task->callback_func = callback;
 	task->callback_arg = (void *)tag;
-	task->cl = &dummy_codelet;
+	task->cl = &starpu_codelet_nop;
 	task->cl_arg = NULL;
 	task->destroy = 0; /* tell StarPU to not destroy the task */
 	task->use_tag = 1;
