@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,22 +25,6 @@
  * Test tag dependencies and waiting for a tag
  */
 
-void dummy_func(void *descr[], void *arg)
-{
-	(void)descr;
-	(void)arg;
-}
-
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {dummy_func},
-	.cuda_funcs = {dummy_func},
-	.opencl_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.model = NULL,
-	.nbuffers = 0
-};
-
 static void callback(void *tag)
 {
 	fflush(stderr);
@@ -52,7 +36,7 @@ static struct starpu_task *create_dummy_task(starpu_tag_t tag)
 {
 	struct starpu_task *task = starpu_task_create();
 
-	task->cl = &dummy_codelet;
+	task->cl = &starpu_codelet_nop;
 	task->cl_arg = NULL;
 	task->callback_func = callback;
 	task->callback_arg = (void *)(uintptr_t)tag;
