@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Simon Archipoff
+ * Copyright (C) 2013-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013	    Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,26 +33,23 @@
 #define NTASKS 1000
 #endif
 
-void
-A(void *buffers[], void *args)
+void funcA(void *buffers[], void *args)
 {
 	(void) buffers;
 	(void) args;
 	FPRINTF(stdout,"A");
-	usleep(1000);
+	starpu_usleep(1000);
 }
 
-void
-B(void *buffers[], void *args)
+void funcB(void *buffers[], void *args)
 {
 	(void) buffers;
 	(void) args;
 	FPRINTF(stdout,"B");
-	usleep(1000);
+	starpu_usleep(1000);
 }
 
-static int
-run(struct starpu_sched_policy *policy)
+static int run(struct starpu_sched_policy *policy)
 {
 	int ret;
 	struct starpu_conf conf;
@@ -67,23 +64,27 @@ run(struct starpu_sched_policy *policy)
 
 	struct starpu_codelet clA =
 	{
-		.cpu_funcs = {A},
-		.cpu_funcs_name = {"A"},
-		.opencl_funcs = {A},
-		.cuda_funcs = {A},
+		.cpu_funcs = {funcA},
+		.cpu_funcs_name = {"funcA"},
+		.opencl_funcs = {funcA},
+		.cuda_funcs = {funcA},
+		.max_fpga_funcs = {funcA},
+		.hip_funcs = {funcA},
 		.nbuffers = 0
 	};
 
 	struct starpu_codelet clB =
 	{
-		.cpu_funcs = {B},
-		.cpu_funcs_name = {"B"},
-		.opencl_funcs = {B},
-		.cuda_funcs = {B},
+		.cpu_funcs = {funcB},
+		.cpu_funcs_name = {"funcB"},
+		.opencl_funcs = {funcB},
+		.cuda_funcs = {funcB},
+		.max_fpga_funcs = {funcB},
+		.hip_funcs = {funcB},
 		.nbuffers = 0
 	};
 
-        starpu_srand48(0);
+	starpu_srand48(0);
 
 	for (i = 0; i < NTASKS; i++)
 	{
